@@ -6,26 +6,25 @@
 **
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
-** No Commercial Usage
-**
-** This file contains pre-release code and may not be distributed.
-** You may use this file in accordance with the terms and conditions
-** contained in the Technology Preview License Agreement accompanying
-** this package.
 **
 ** GNU Lesser General Public License Usage
 **
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this file.
+** Please review the following information to ensure the GNU Lesser General
+** Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
+** Other Usage
+**
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
 ** If you have questions regarding the use of this file, please contact
 ** Nokia at qt-info@nokia.com.
 **
@@ -44,8 +43,14 @@
 #include <Symbols.h>
 #include <Names.h>
 #include <AST.h>
+#include <ASTPatternBuilder.h>
+#include <ASTMatcher.h>
 #include <Scope.h>
 #include <SymbolVisitor.h>
+#include <NameVisitor.h>
+#include <TypeVisitor.h>
+#include <CoreTypes.h>
+#include <LookupContext.h>
 
 #include <QtCore/QByteArray>
 #include <QtCore/QBitArray>
@@ -138,7 +143,7 @@ protected:
                 unsigned endLine, endColumn;
                 _unit->getPosition(scope->endOffset(), &endLine, &endColumn);
 
-                if (_line < endLine || (_line == endLine && _column <= endColumn))
+                if (_line < endLine || (_line == endLine && _column < endColumn))
                     _scope = scope;
             }
         }
@@ -251,7 +256,8 @@ Document::Document(const QString &fileName)
     : _fileName(QDir::cleanPath(fileName)),
       _globalNamespace(0),
       _revision(0),
-      _editorRevision(0)
+      _editorRevision(0),
+      _checkMode(0)
 {
     _control = new Control();
 
@@ -564,6 +570,8 @@ void Document::check(CheckMode mode)
 {
     Q_ASSERT(!_globalNamespace);
 
+    _checkMode = mode;
+
     if (! isParsed())
         parse();
 
@@ -579,17 +587,39 @@ void Document::check(CheckMode mode)
         semantic(ast, _globalNamespace);
     } else if (ExpressionAST *ast = _translationUnit->ast()->asExpression()) {
         semantic(ast, _globalNamespace);
+    } else if (DeclarationAST *ast = translationUnit()->ast()->asDeclaration()) {
+        semantic(ast, _globalNamespace);
     }
 }
 
-void Document::releaseSource()
+void Document::keepSourceAndAST()
 {
-    _source.clear();
+    _keepSourceAndASTCount.ref();
 }
 
-void Document::releaseTranslationUnit()
+void Document::releaseSourceAndAST()
 {
-    _translationUnit->release();
+    if (!_keepSourceAndASTCount.deref()) {
+        _source.clear();
+        _translationUnit->release();
+        _control->squeeze();
+    }
+}
+
+bool Document::DiagnosticMessage::operator==(const Document::DiagnosticMessage &other) const
+{
+    return
+            _line == other._line &&
+            _column == other._column &&
+            _length == other._length &&
+            _level == other._level &&
+            _fileName == other._fileName &&
+            _text == other._text;
+}
+
+bool Document::DiagnosticMessage::operator!=(const Document::DiagnosticMessage &other) const
+{
+    return !operator==(other);
 }
 
 Snapshot::Snapshot()
@@ -691,11 +721,17 @@ namespace {
 class FindMatchingDefinition: public SymbolVisitor
 {
     Symbol *_declaration;
+    const OperatorNameId *_oper;
     QList<Function *> _result;
 
 public:
     FindMatchingDefinition(Symbol *declaration)
-        : _declaration(declaration) {}
+        : _declaration(declaration)
+        , _oper(0)
+    {
+        if (_declaration->name())
+            _oper = _declaration->name()->asOperatorNameId();
+    }
 
     QList<Function *> result() const { return _result; }
 
@@ -703,8 +739,15 @@ public:
 
     virtual bool visit(Function *fun)
     {
-        if (_declaration->identifier()->isEqualTo(fun->identifier()))
-            _result.append(fun);
+        if (_oper) {
+            if (const Name *name = fun->unqualifiedName()) {
+                    if (_oper->isEqualTo(name))
+                        _result.append(fun);
+            }
+        } else if (const Identifier *id = _declaration->identifier()) {
+            if (id->isEqualTo(fun->identifier()))
+                _result.append(fun);
+        }
 
         return false;
     }
@@ -716,9 +759,11 @@ public:
 };
 } // end of anonymous namespace
 
-Symbol *Snapshot::findMatchingDefinition(Symbol *declaration) const
+// strict means the returned symbol has to match exactly,
+// including argument count and argument types
+Symbol *Snapshot::findMatchingDefinition(Symbol *declaration, bool strict) const
 {
-    if (! (declaration && declaration->identifier()))
+    if (!declaration)
         return 0;
 
     Document::Ptr thisDocument = document(QString::fromUtf8(declaration->fileName(), declaration->fileNameLength()));
@@ -734,9 +779,19 @@ Symbol *Snapshot::findMatchingDefinition(Symbol *declaration) const
     }
 
     foreach (Document::Ptr doc, *this) {
-        if (! doc->control()->findIdentifier(declaration->identifier()->chars(),
-                                             declaration->identifier()->size()))
+        const Identifier *id = declaration->identifier();
+        if (id && ! doc->control()->findIdentifier(id->chars(),
+                                                   id->size()))
             continue;
+        if (!id) {
+            if (!declaration->name())
+                continue;
+            const OperatorNameId *oper = declaration->name()->asOperatorNameId();
+            if (!oper)
+                continue;
+            if (!doc->control()->findOperatorNameId(oper->kind()))
+                continue;
+        }
 
         FindMatchingDefinition candidates(declaration);
         candidates.accept(doc->globalNamespace());
@@ -764,17 +819,16 @@ Symbol *Snapshot::findMatchingDefinition(Symbol *declaration) const
             if (viableFunctions.isEmpty())
                 continue;
 
-            else if (viableFunctions.length() == 1)
+            else if (! strict && viableFunctions.length() == 1)
                 return viableFunctions.first();
 
             Function *best = 0;
 
             foreach (Function *fun, viableFunctions) {
-                if (fun->unqualifiedName()->isEqualTo(declaration->unqualifiedName()))
+                if (! (fun->unqualifiedName() && fun->unqualifiedName()->isEqualTo(declaration->unqualifiedName())))
                     continue;
-
                 else if (fun->argumentCount() == declarationTy->argumentCount()) {
-                    if (! best)
+                    if (! strict && ! best)
                         best = fun;
 
                     unsigned argc = 0;
@@ -790,9 +844,11 @@ Symbol *Snapshot::findMatchingDefinition(Symbol *declaration) const
                 }
             }
 
+            if (strict && ! best)
+                continue;
+
             if (! best)
                 best = viableFunctions.first();
-
             return best;
         }
     }
@@ -824,3 +880,76 @@ Class *Snapshot::findMatchingClassDeclaration(Symbol *declaration) const
 
     return 0;
 }
+
+void CPlusPlus::findMatchingDeclaration(const LookupContext &context,
+                                        Function *functionType,
+                                        QList<Declaration *> *typeMatch,
+                                        QList<Declaration *> *argumentCountMatch,
+                                        QList<Declaration *> *nameMatch)
+{
+    Scope *enclosingScope = functionType->enclosingScope();
+    while (! (enclosingScope->isNamespace() || enclosingScope->isClass()))
+        enclosingScope = enclosingScope->enclosingScope();
+    Q_ASSERT(enclosingScope != 0);
+
+    const Name *functionName = functionType->name();
+    if (! functionName)
+        return; // anonymous function names are not valid c++
+
+    ClassOrNamespace *binding = 0;
+    const QualifiedNameId *qName = functionName->asQualifiedNameId();
+    if (qName) {
+        if (qName->base())
+            binding = context.lookupType(qName->base(), enclosingScope);
+        else
+            binding = context.globalNamespace();
+        functionName = qName->name();
+    }
+
+    if (!binding) { // declaration for a global function
+        binding = context.lookupType(enclosingScope);
+
+        if (!binding)
+            return;
+    }
+
+    const Identifier *funcId = functionName->identifier();
+    if (!funcId) // E.g. operator, which we might be able to handle in the future...
+        return;
+
+    foreach (Symbol *s, binding->symbols()) {
+        Scope *scope = s->asScope();
+        if (!scope)
+            continue;
+
+        for (Symbol *s = scope->find(funcId); s; s = s->next()) {
+            if (! s->name())
+                continue;
+            else if (! funcId->isEqualTo(s->identifier()))
+                continue;
+            else if (! s->type()->isFunctionType())
+                continue;
+            else if (Declaration *decl = s->asDeclaration()) {
+                if (Function *declFunTy = decl->type()->asFunctionType()) {
+                    if (functionType->isEqualTo(declFunTy))
+                        typeMatch->prepend(decl);
+                    else if (functionType->argumentCount() == declFunTy->argumentCount())
+                        argumentCountMatch->prepend(decl);
+                    else
+                        nameMatch->append(decl);
+                }
+            }
+        }
+    }
+}
+
+QList<Declaration *> CPlusPlus::findMatchingDeclaration(const LookupContext &context, Function *functionType)
+{
+    QList<Declaration *> result;
+    QList<Declaration *> nameMatch, argumentCountMatch, typeMatch;
+    findMatchingDeclaration(context, functionType, &typeMatch, &argumentCountMatch, &nameMatch);
+    result.append(typeMatch);
+    result.append(argumentCountMatch);
+    result.append(nameMatch);
+    return result;
+}
