@@ -6,26 +6,25 @@
 **
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
-** No Commercial Usage
-**
-** This file contains pre-release code and may not be distributed.
-** You may use this file in accordance with the terms and conditions
-** contained in the Technology Preview License Agreement accompanying
-** this package.
 **
 ** GNU Lesser General Public License Usage
 **
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this file.
+** Please review the following information to ensure the GNU Lesser General
+** Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
+** Other Usage
+**
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
 ** If you have questions regarding the use of this file, please contact
 ** Nokia at qt-info@nokia.com.
 **
@@ -272,7 +271,7 @@ void ManhattanStyle::polish(QWidget *widget)
     QProxyStyle::polish(widget);
 
     // OxygenStyle forces a rounded widget mask on toolbars and dock widgets
-    if (baseStyle()->inherits("OxygenStyle")) {
+    if (baseStyle()->inherits("OxygenStyle") || baseStyle()->inherits("Oxygen::Style")) {
         if (qobject_cast<QToolBar*>(widget) || qobject_cast<QDockWidget*>(widget)) {
             widget->removeEventFilter(baseStyle());
             widget->setContentsMargins(0, 0, 0, 0);
@@ -565,8 +564,8 @@ void ManhattanStyle::drawPrimitive(PrimitiveElement element, const QStyleOption
             bool horizontal = option->state & State_Horizontal;
             painter->save();
             QPainterPath path;
-            int x = option->rect.x() + horizontal ? 2 : 6;
-            int y = option->rect.y() + horizontal ? 6 : 2;
+            int x = option->rect.x() + (horizontal ? 2 : 6);
+            int y = option->rect.y() + (horizontal ? 6 : 2);
             static const int RectHeight = 2;
             if (horizontal) {
                 while (y < option->rect.height() - RectHeight - 6) {
