@@ -6,26 +6,25 @@
 **
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
-** No Commercial Usage
-**
-** This file contains pre-release code and may not be distributed.
-** You may use this file in accordance with the terms and conditions
-** contained in the Technology Preview License Agreement accompanying
-** this package.
 **
 ** GNU Lesser General Public License Usage
 **
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this file.
+** Please review the following information to ensure the GNU Lesser General
+** Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
+** Other Usage
+**
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
 ** If you have questions regarding the use of this file, please contact
 ** Nokia at qt-info@nokia.com.
 **
@@ -48,7 +47,7 @@
 #include <coreplugin/editormanager/editormanager.h>
 #include <coreplugin/findplaceholder.h>
 #include <coreplugin/editormanager/ieditor.h>
-#include <coreplugin/uniqueidmanager.h>
+#include <coreplugin/id.h>
 
 #include <extensionsystem/pluginmanager.h>
 
@@ -98,6 +97,8 @@ void OutputPaneManager::updateStatusButtons(bool visible)
     int idx = m_widgetComboBox->itemData(m_widgetComboBox->currentIndex()).toInt();
     if (m_buttons.value(idx))
         m_buttons.value(idx)->setChecked(visible);
+    if (m_pageMap.value(idx))
+        m_pageMap.value(idx)->visibilityChanged(visible);
     m_minMaxAction->setVisible(OutputPanePlaceHolder::getCurrent()
                                && OutputPanePlaceHolder::getCurrent()->canMaximizeOrMinimize());
 }
@@ -105,7 +106,6 @@ void OutputPaneManager::updateStatusButtons(bool visible)
 OutputPaneManager::OutputPaneManager(QWidget *parent) :
     QWidget(parent),
     m_widgetComboBox(new QComboBox),
-    m_clearButton(new QToolButton),
     m_closeButton(new QToolButton),
     m_minMaxAction(0),
     m_minMaxButton(new QToolButton),
@@ -121,9 +121,10 @@ OutputPaneManager::OutputPaneManager(QWidget *parent) :
     setWindowTitle(tr("Output"));
     connect(m_widgetComboBox, SIGNAL(currentIndexChanged(int)), this, SLOT(changePage()));
 
-    m_clearButton->setIcon(QIcon(QLatin1String(Constants::ICON_CLEAN_PANE)));
-    m_clearButton->setToolTip(tr("Clear"));
-    connect(m_clearButton, SIGNAL(clicked()), this, SLOT(clearPage()));
+    m_clearAction = new QAction(this);
+    m_clearAction->setIcon(QIcon(QLatin1String(Constants::ICON_CLEAN_PANE)));
+    m_clearAction->setText(tr("Clear"));
+    connect(m_clearAction, SIGNAL(triggered()), this, SLOT(clearPage()));
 
     m_nextAction = new QAction(this);
     m_nextAction->setIcon(QIcon(QLatin1String(Constants::ICON_NEXT)));
@@ -150,6 +151,7 @@ OutputPaneManager::OutputPaneManager(QWidget *parent) :
     toolLayout->setMargin(0);
     toolLayout->setSpacing(0);
     toolLayout->addWidget(m_widgetComboBox);
+    m_clearButton = new QToolButton;
     toolLayout->addWidget(m_clearButton);
     m_prevToolButton = new QToolButton;
     toolLayout->addWidget(m_prevToolButton);
@@ -205,6 +207,10 @@ void OutputPaneManager::init()
 
     Core::Command *cmd;
 
+    cmd = am->registerAction(m_clearAction, "Coreplugin.OutputPane.clear", globalcontext);
+    m_clearButton->setDefaultAction(cmd->action());
+    mpanes->addAction(cmd, "Coreplugin.OutputPane.ActionsGroup");
+
     cmd = am->registerAction(m_prevAction, "Coreplugin.OutputPane.previtem", globalcontext);
     cmd->setDefaultKeySequence(QKeySequence("Shift+F6"));
     m_prevToolButton->setDefaultAction(cmd->action());
@@ -248,7 +254,7 @@ void OutputPaneManager::init()
         const int idx = m_outputWidgetPane->addWidget(outPane->outputWidget(this));
 
         m_pageMap.insert(idx, outPane);
-        connect(outPane, SIGNAL(showPage(bool)), this, SLOT(showPage(bool)));
+        connect(outPane, SIGNAL(showPage(bool,bool)), this, SLOT(showPage(bool,bool)));
         connect(outPane, SIGNAL(hidePage()), this, SLOT(slotHide()));
         connect(outPane, SIGNAL(togglePage(bool)), this, SLOT(togglePage(bool)));
         connect(outPane, SIGNAL(navigateStateUpdate()), this, SLOT(updateNavigateState()));
@@ -275,7 +281,7 @@ void OutputPaneManager::init()
 
         if (outPane->priorityInStatusBar() != -1) {
             cmd->setDefaultKeySequence(QKeySequence(paneShortCut(shortcutNumber)));
-            QPushButton *button = new OutputPaneToggleButton(shortcutNumber, outPane->displayName(),
+            QToolButton *button = new OutputPaneToggleButton(shortcutNumber, outPane->displayName(),
                                                              cmd->action());
             ++shortcutNumber;
             m_buttonsWidget->layout()->addWidget(button);
@@ -335,8 +341,8 @@ void OutputPaneManager::slotMinMax()
 
 void OutputPaneManager::buttonTriggered()
 {
-    QPushButton *button = qobject_cast<QPushButton *>(sender());
-    QMap<int, QPushButton *>::const_iterator it, end;
+    QToolButton *button = qobject_cast<QToolButton *>(sender());
+    QMap<int, QToolButton *>::const_iterator it, end;
     end = m_buttons.constEnd();
     for (it = m_buttons.begin(); it != end; ++it) {
         if (it.value() == button)
@@ -345,8 +351,7 @@ void OutputPaneManager::buttonTriggered()
     int idx = it.key();
 
     if (m_widgetComboBox->itemData(m_widgetComboBox->currentIndex()).toInt() == idx &&
-        OutputPanePlaceHolder::isCurrentVisible()
-        && OutputPanePlaceHolder::getCurrent()->closeable()) {
+        OutputPanePlaceHolder::isCurrentVisible()) {
         // we should toggle and the page is already visible and we are actually closeable
         slotHide();
     } else {
@@ -379,6 +384,8 @@ void OutputPaneManager::slotHide()
         int idx = m_widgetComboBox->itemData(m_widgetComboBox->currentIndex()).toInt();
         if (m_buttons.value(idx))
             m_buttons.value(idx)->setChecked(false);
+        if (m_pageMap.value(idx))
+            m_pageMap.value(idx)->visibilityChanged(false);
         if (IEditor *editor = Core::EditorManager::instance()->currentEditor())
             editor->widget()->setFocus();
     }
@@ -424,10 +431,12 @@ void OutputPaneManager::updateNavigateState()
 }
 
 // Slot connected to showPage signal of each page
-void OutputPaneManager::showPage(bool focus)
+void OutputPaneManager::showPage(bool focus, bool ensureSizeHint)
 {
     int idx = findIndexForPage(qobject_cast<IOutputPane*>(sender()));
     showPage(idx, focus);
+    if (ensureSizeHint && OutputPanePlaceHolder::getCurrent())
+        OutputPanePlaceHolder::getCurrent()->ensureSizeHintAsMinimum();
 }
 
 void OutputPaneManager::showPage(int idx, bool focus)
@@ -437,7 +446,7 @@ void OutputPaneManager::showPage(int idx, bool focus)
         if (!OutputPanePlaceHolder::getCurrent()) {
             // In this mode we don't have a placeholder
             // switch to the output mode and switch the page
-            ICore::instance()->modeManager()->activateMode(Constants::MODE_EDIT);
+            ModeManager::instance()->activateMode(Constants::MODE_EDIT);
         }
         if (OutputPanePlaceHolder::getCurrent()) {
             // make the page visible
@@ -460,16 +469,6 @@ void OutputPaneManager::togglePage(bool focus)
     }
 }
 
-void OutputPaneManager::setCloseable(bool b)
-{
-    m_closeButton->setVisible(b);
-}
-
-bool OutputPaneManager::closeable()
-{
-    return m_closeButton->isVisibleTo(m_closeButton->parentWidget());
-}
-
 void OutputPaneManager::focusInEvent(QFocusEvent *e)
 {
     if (m_outputWidgetPane->currentWidget())
@@ -521,22 +520,24 @@ void OutputPaneManager::clearPage()
 
 OutputPaneToggleButton::OutputPaneToggleButton(int number, const QString &text,
                                                QAction *action, QWidget *parent)
-    : QPushButton(parent)
+    : QToolButton(parent)
     , m_number(QString::number(number))
     , m_text(text)
     , m_action(action)
 {
     setFocusPolicy(Qt::NoFocus);
     setCheckable(true);
+    QFont fnt = QApplication::font();
+    setFont(fnt);
     setStyleSheet(
-            "QPushButton { border-image: url(:/core/images/panel_button.png) 2 2 2 19;"
+            "QToolButton { border-image: url(:/core/images/panel_button.png) 2 2 2 19;"
                          " border-width: 2px 2px 2px 19px; padding-left: -17; padding-right: 4 } "
-            "QPushButton:checked { border-image: url(:/core/images/panel_button_checked.png) 2 2 2 19 } "
-            "QPushButton::menu-indicator { width:0; height:0 }"
+            "QToolButton:checked { border-image: url(:/core/images/panel_button_checked.png) 2 2 2 19 } "
+            "QToolButton::menu-indicator { width:0; height:0 }"
 #ifndef Q_WS_MAC // Mac UIs usually don't hover
-            "QPushButton:checked:hover { border-image: url(:/core/images/panel_button_checked_hover.png) 2 2 2 19 } "
-            "QPushButton:pressed:hover { border-image: url(:/core/images/panel_button_pressed.png) 2 2 2 19 } "
-            "QPushButton:hover { border-image: url(:/core/images/panel_button_hover.png) 2 2 2 19 } "
+            "QToolButton:checked:hover { border-image: url(:/core/images/panel_button_checked_hover.png) 2 2 2 19 } "
+            "QToolButton:pressed:hover { border-image: url(:/core/images/panel_button_pressed.png) 2 2 2 19 } "
+            "QToolButton:hover { border-image: url(:/core/images/panel_button_hover.png) 2 2 2 19 } "
 #endif
             );
     if (m_action)
@@ -565,7 +566,7 @@ QSize OutputPaneToggleButton::sizeHint() const
 void OutputPaneToggleButton::paintEvent(QPaintEvent *event)
 {
     // For drawing the style sheet stuff
-    QPushButton::paintEvent(event);
+    QToolButton::paintEvent(event);
 
     const QFontMetrics fm = fontMetrics();
     const int baseLine = (height() - fm.height() + 1) / 2 + fm.ascent();
