@@ -6,26 +6,25 @@
 **
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
-** No Commercial Usage
-**
-** This file contains pre-release code and may not be distributed.
-** You may use this file in accordance with the terms and conditions
-** contained in the Technology Preview License Agreement accompanying
-** this package.
 **
 ** GNU Lesser General Public License Usage
 **
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this file.
+** Please review the following information to ensure the GNU Lesser General
+** Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
+** Other Usage
+**
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
 ** If you have questions regarding the use of this file, please contact
 ** Nokia at qt-info@nokia.com.
 **
@@ -34,6 +33,7 @@
 #include "findtoolbar.h"
 #include "findplugin.h"
 #include "textfindconstants.h"
+#include "ifindfilter.h"
 
 #include <coreplugin/coreconstants.h>
 #include <coreplugin/icontext.h>
@@ -42,11 +42,12 @@
 #include <coreplugin/actionmanager/actioncontainer.h>
 #include <coreplugin/actionmanager/command.h>
 #include <coreplugin/findplaceholder.h>
-#include <coreplugin/uniqueidmanager.h>
+#include <coreplugin/id.h>
 
 #include <extensionsystem/pluginmanager.h>
 
 #include <utils/stylehelper.h>
+#include <utils/flowlayout.h>
 
 #include <QtCore/QDebug>
 #include <QtCore/QSettings>
@@ -79,15 +80,20 @@ FindToolBar::FindToolBar(FindPlugin *plugin, CurrentDocumentFind *currentDocumen
       m_replaceAction(0),
       m_replaceNextAction(0),
       m_replacePreviousAction(0),
-      m_casesensitiveIcon(":/find/images/casesensitively.png"),
-      m_regexpIcon(":/find/images/regexp.png"),
-      m_wholewordsIcon(":/find/images/wholewords.png"),
       m_findIncrementalTimer(this), m_findStepTimer(this),
       m_useFakeVim(false),
       m_eventFiltersInstalled(false)
 {
     //setup ui
     m_ui.setupUi(this);
+    // compensate for a vertically expanding spacer below the label
+    m_ui.replaceLabel->setMinimumHeight(m_ui.replaceEdit->sizeHint().height());
+    delete m_ui.replaceButtonsWidget->layout();
+    Utils::FlowLayout *flowlayout = new Utils::FlowLayout(m_ui.replaceButtonsWidget, 0, 3, 3);
+    flowlayout->addWidget(m_ui.replaceButton);
+    flowlayout->addWidget(m_ui.replaceNextButton);
+    flowlayout->addWidget(m_ui.replaceAllButton);
+    m_ui.replaceButtonsWidget->setLayout(flowlayout);
     setFocusProxy(m_ui.findEdit);
     setProperty("topBorder", true);
     setSingleRow(false);
@@ -349,9 +355,7 @@ void FindToolBar::updateToolBar()
     m_ui.replaceLabel->setEnabled(replaceEnabled);
     m_ui.replaceEdit->setVisible(replaceEnabled);
     m_ui.replaceLabel->setVisible(replaceEnabled);
-    m_ui.replaceButton->setVisible(replaceEnabled);
-    m_ui.replaceNextButton->setVisible(replaceEnabled);
-    m_ui.replaceAllButton->setVisible(replaceEnabled);
+    m_ui.replaceButtonsWidget->setVisible(replaceEnabled);
     m_ui.advancedButton->setVisible(replaceEnabled);
     layout()->invalidate();
 
@@ -533,33 +537,17 @@ void FindToolBar::updateIcons()
     bool casesensitive = effectiveFlags & Find::FindCaseSensitively;
     bool wholewords = effectiveFlags & Find::FindWholeWords;
     bool regexp = effectiveFlags & Find::FindRegularExpression;
-    int width = 0;
-    if (casesensitive) width += 6;
-    if (wholewords) width += 6;
-    if (regexp) width += 6;
-    if (width == 0) width = 18;
-    --width;
-    QPixmap pixmap(width, 17);
-    pixmap.fill(Qt::transparent);
-    QPainter painter(&pixmap);
-    int x = 0;
-
-    if (casesensitive) {
-        painter.drawPixmap(x - 6, 0, m_casesensitiveIcon);
-        x += 6;
-    }
-    if (wholewords) {
-        painter.drawPixmap(x - 6, 0, m_wholewordsIcon);
-        x += 6;
-    }
-    if (regexp) {
-        painter.drawPixmap(x - 6, 0, m_regexpIcon);
-    }
     if (!casesensitive && !wholewords && !regexp) {
+        QPixmap pixmap(17, 17);
+        pixmap.fill(Qt::transparent);
+        QPainter painter(&pixmap);
         QPixmap mag(Core::Constants::ICON_MAGNIFIER);
         painter.drawPixmap(0, (pixmap.height() - mag.height()) / 2, mag);
+        m_ui.findEdit->setButtonPixmap(Utils::FancyLineEdit::Left, pixmap);
+    } else {
+        m_ui.findEdit->setButtonPixmap(Utils::FancyLineEdit::Left,
+                                       IFindFilter::pixmapForFindFlags(effectiveFlags));
     }
-    m_ui.findEdit->setButtonPixmap(Utils::FancyLineEdit::Left, pixmap);
 }
 
 Find::FindFlags FindToolBar::effectiveFindFlags()
