@@ -1,3 +1,35 @@
+/**************************************************************************
+**
+** This file is part of Qt Creator
+**
+** Copyright (c) 2011 Nokia Corporation and/or its subsidiary(-ies).
+**
+** Contact: Nokia Corporation (qt-info@nokia.com)
+**
+**
+** GNU Lesser General Public License Usage
+**
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this file.
+** Please review the following information to ensure the GNU Lesser General
+** Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+**
+** In addition, as a special exception, Nokia gives you certain additional
+** rights. These rights are described in the Nokia Qt LGPL Exception
+** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+**
+** Other Usage
+**
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
+** If you have questions regarding the use of this file, please contact
+** Nokia at qt-info@nokia.com.
+**
+**************************************************************************/
+
 #include "librarydetailscontroller.h"
 #include "ui_librarydetailswidget.h"
 #include "findqt4profiles.h"
@@ -6,7 +38,9 @@
 
 #include <projectexplorer/projectexplorer.h>
 #include <projectexplorer/session.h>
+#include <projectexplorer/project.h>
 #include <projectexplorer/target.h>
+#include <projectexplorer/toolchain.h>
 #include <projectexplorer/buildconfiguration.h>
 
 #include <QtCore/QFileInfo>
@@ -52,11 +86,19 @@ LibraryDetailsController::LibraryDetailsController(
     // take active build configuration for it
     Qt4BuildConfiguration *qt4BuildConfiguration =
             qobject_cast<Qt4BuildConfiguration *>(project->activeTarget()->activeBuildConfiguration());
-    // if its toolchain is maemo behave the same as we would be on linux
-    if (qt4BuildConfiguration && qt4BuildConfiguration->toolChainType() == ProjectExplorer::ToolChain::GCC_MAEMO)
+    // if its tool chain is maemo behave the same as we would be on linux
+    if (qt4BuildConfiguration
+            && qt4BuildConfiguration->toolChain()
+            && (qt4BuildConfiguration->toolChain()->targetAbi().osFlavor() == ProjectExplorer::Abi::HarmattanLinuxFlavor
+                || qt4BuildConfiguration->toolChain()->targetAbi().osFlavor() == ProjectExplorer::Abi::MaemoLinuxFlavor))
         m_creatorPlatform = CreatorLinux;
 #endif
 
+    setPlatformsVisible(true);
+    setLinkageGroupVisible(true);
+    setMacLibraryGroupVisible(true);
+    setPackageLineEditVisible(false);
+
     if (creatorPlatform() == CreatorMac)
         setMacLibraryRadiosVisible(false);
 
@@ -230,6 +272,11 @@ void LibraryDetailsController::showMacLibraryType(
     libraryDetailsWidget()->macGroupBox->setTitle(libraryTypeTitle);
 }
 
+void LibraryDetailsController::setPlatformsVisible(bool ena)
+{
+    libraryDetailsWidget()->platformGroupBox->setVisible(ena);
+}
+
 void LibraryDetailsController::setLinkageRadiosVisible(bool ena)
 {
     m_linkageRadiosVisible = ena;
@@ -237,6 +284,12 @@ void LibraryDetailsController::setLinkageRadiosVisible(bool ena)
     libraryDetailsWidget()->dynamicRadio->setVisible(ena);
 }
 
+void LibraryDetailsController::setLinkageGroupVisible(bool ena)
+{
+    setLinkageRadiosVisible(ena);
+    libraryDetailsWidget()->linkageGroupBox->setVisible(ena);
+}
+
 void LibraryDetailsController::setMacLibraryRadiosVisible(bool ena)
 {
     m_macLibraryRadiosVisible = ena;
@@ -244,6 +297,12 @@ void LibraryDetailsController::setMacLibraryRadiosVisible(bool ena)
     libraryDetailsWidget()->libraryRadio->setVisible(ena);
 }
 
+void LibraryDetailsController::setMacLibraryGroupVisible(bool ena)
+{
+    setMacLibraryRadiosVisible(ena);
+    libraryDetailsWidget()->macGroupBox->setVisible(ena);
+}
+
 void LibraryDetailsController::setLibraryPathChooserVisible(bool ena)
 {
     libraryDetailsWidget()->libraryPathChooser->setVisible(ena);
@@ -256,6 +315,12 @@ void LibraryDetailsController::setLibraryComboBoxVisible(bool ena)
     libraryDetailsWidget()->libraryLabel->setVisible(ena);
 }
 
+void LibraryDetailsController::setPackageLineEditVisible(bool ena)
+{
+    libraryDetailsWidget()->packageLineEdit->setVisible(ena);
+    libraryDetailsWidget()->packageLabel->setVisible(ena);
+}
+
 void LibraryDetailsController::setIncludePathVisible(bool ena)
 {
     m_includePathVisible = ena;
@@ -737,6 +802,65 @@ QString NonInternalLibraryDetailsController::snippet() const
 
 /////////////
 
+PackageLibraryDetailsController::PackageLibraryDetailsController(
+    Ui::LibraryDetailsWidget *libraryDetails,
+    const QString &proFile, QObject *parent)
+    : NonInternalLibraryDetailsController(libraryDetails, proFile, parent)
+{
+    setPlatformsVisible(false);
+    setIncludePathVisible(false);
+    setWindowsGroupVisible(false);
+    setLinkageGroupVisible(false);
+    setMacLibraryGroupVisible(false);
+    setLibraryPathChooserVisible(false);
+    setPackageLineEditVisible(true);
+
+    connect(libraryDetailsWidget()->packageLineEdit, SIGNAL(textChanged(QString)),
+            this, SIGNAL(completeChanged()));
+
+    updateGui();
+}
+
+bool PackageLibraryDetailsController::isComplete() const
+{
+    return !libraryDetailsWidget()->packageLineEdit->text().isEmpty();
+}
+
+QString PackageLibraryDetailsController::snippet() const
+{
+    QString snippetMessage;
+    QTextStream str(&snippetMessage);
+    str << "\n";
+    if (!isLinkPackageGenerated())
+        str << "unix: CONFIG += link_pkgconfig\n";
+    str << "unix: PKGCONFIG += " << libraryDetailsWidget()->packageLineEdit->text() << "\n";
+    return snippetMessage;
+}
+
+bool PackageLibraryDetailsController::isLinkPackageGenerated() const
+{
+    const ProjectExplorer::Project *project =
+            ProjectExplorer::ProjectExplorerPlugin::instance()->session()->projectForFile(proFile());
+    if (!project)
+        return false;
+
+    const Qt4ProFileNode *rootProject = qobject_cast<const Qt4ProFileNode *>(project->rootProjectNode());
+    if (!rootProject)
+        return false;
+
+    const Qt4ProFileNode *currentProject = rootProject->findProFileFor(proFile());
+    if (!currentProject)
+        return false;
+
+    const QStringList configVar = currentProject->variableValue(ConfigVar);
+    if (configVar.contains(QLatin1String("link_pkgconfig")))
+        return true;
+
+    return false;
+}
+
+/////////////
+
 SystemLibraryDetailsController::SystemLibraryDetailsController(
     Ui::LibraryDetailsWidget *libraryDetails,
     const QString &proFile, QObject *parent)
