@@ -6,26 +6,25 @@
 **
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
-** No Commercial Usage
-**
-** This file contains pre-release code and may not be distributed.
-** You may use this file in accordance with the terms and conditions
-** contained in the Technology Preview License Agreement accompanying
-** this package.
 **
 ** GNU Lesser General Public License Usage
 **
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this file.
+** Please review the following information to ensure the GNU Lesser General
+** Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
+** Other Usage
+**
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
 ** If you have questions regarding the use of this file, please contact
 ** Nokia at qt-info@nokia.com.
 **
@@ -33,33 +32,42 @@
 
 #include "qt4project.h"
 
-#include "profilereader.h"
 #include "qt4projectmanager.h"
+#include "qt4target.h"
 #include "makestep.h"
 #include "qmakestep.h"
-#include "qt4runconfiguration.h"
 #include "qt4nodes.h"
 #include "qt4projectconfigwidget.h"
 #include "qt4projectmanagerconstants.h"
 #include "projectloadwizard.h"
 #include "qt4buildconfiguration.h"
 #include "findqt4profiles.h"
-#include "qmldumptool.h"
 
 #include <coreplugin/icore.h>
+#include <coreplugin/ifile.h>
+#include <coreplugin/icontext.h>
 #include <coreplugin/messagemanager.h>
 #include <coreplugin/coreconstants.h>
 #include <coreplugin/progressmanager/progressmanager.h>
 #include <extensionsystem/pluginmanager.h>
-#include <cpptools/cppmodelmanagerinterface.h>
+#include <cplusplus/ModelManagerInterface.h>
 #include <qmljs/qmljsmodelmanagerinterface.h>
+#include <projectexplorer/toolchain.h>
+#include <projectexplorer/headerpath.h>
+#include <projectexplorer/target.h>
 #include <projectexplorer/buildenvironmentwidget.h>
 #include <projectexplorer/customexecutablerunconfiguration.h>
 #include <projectexplorer/projectexplorer.h>
+#include <projectexplorer/projectexplorerconstants.h>
 #include <utils/qtcassert.h>
+#include <qtsupport/qmldumptool.h>
+#include <qtsupport/baseqtversion.h>
+#include <qtsupport/profilereader.h>
+#include <qtsupport/qtsupportconstants.h>
 
 #include <QtCore/QDebug>
 #include <QtCore/QDir>
+#include <QtCore/QFileSystemWatcher>
 #include <QtGui/QFileDialog>
 #include <QtGui/QInputDialog>
 
@@ -72,6 +80,62 @@ enum { debug = 0 };
 namespace Qt4ProjectManager {
 namespace Internal {
 
+class Qt4ProjectFile : public Core::IFile
+{
+    Q_OBJECT
+
+public:
+    Qt4ProjectFile(Qt4Project *project, const QString &filePath, QObject *parent = 0);
+
+    bool save(QString *errorString, const QString &fileName, bool autoSave);
+    QString fileName() const;
+    virtual void rename(const QString &newName);
+
+    QString defaultPath() const;
+    QString suggestedFileName() const;
+    virtual QString mimeType() const;
+
+    bool isModified() const;
+    bool isReadOnly() const;
+    bool isSaveAsAllowed() const;
+
+    ReloadBehavior reloadBehavior(ChangeTrigger state, ChangeType type) const;
+    bool reload(QString *errorString, ReloadFlag flag, ChangeType type);
+
+private:
+    const QString m_mimeType;
+    Qt4Project *m_project;
+    QString m_filePath;
+};
+
+/// Watches folders for Qt4PriFile nodes
+/// use one file system watcher to watch all folders
+/// such minimizing system ressouce usage
+
+class CentralizedFolderWatcher : public QObject
+{
+    Q_OBJECT
+public:
+    CentralizedFolderWatcher(QObject *parent);
+    ~CentralizedFolderWatcher();
+    void watchFolders(const QList<QString> &folders, Qt4ProjectManager::Qt4PriFileNode *node);
+    void unwatchFolders(const QList<QString> &folders, Qt4ProjectManager::Qt4PriFileNode *node);
+
+private slots:
+    void folderChanged(const QString &folder);
+    void onTimer();
+    void delayedFolderChanged(const QString &folder);
+
+private:
+    QSet<QString> recursiveDirs(const QString &folder);
+    QFileSystemWatcher m_watcher;
+    QMultiMap<QString, Qt4ProjectManager::Qt4PriFileNode *> m_map;
+
+    QSet<QString> m_recursiveWatchedFolders;
+    QTimer m_compressTimer;
+    QSet<QString> m_changedFolders;
+};
+
 // Qt4ProjectFiles: Struct for (Cached) lists of files in a project
 struct Qt4ProjectFiles {
     void clear();
@@ -119,8 +183,8 @@ QDebug operator<<(QDebug d, const  Qt4ProjectFiles &f)
 // A visitor to collect all files of a project in a Qt4ProjectFiles struct
 class ProjectFilesVisitor : public ProjectExplorer::NodesVisitor
 {
-    Q_DISABLE_COPY(ProjectFilesVisitor)
     ProjectFilesVisitor(Qt4ProjectFiles *files);
+
 public:
 
     static void findProjectFiles(Qt4ProFileNode *rootNode, Qt4ProjectFiles *files);
@@ -168,10 +232,10 @@ void ProjectFilesVisitor::visitFolderNode(FolderNode *folderNode)
     }
 }
 
-}
 }
 
 // ----------- Qt4ProjectFile
+namespace Internal {
 Qt4ProjectFile::Qt4ProjectFile(Qt4Project *project, const QString &filePath, QObject *parent)
     : Core::IFile(parent),
       m_mimeType(QLatin1String(Qt4ProjectManager::Constants::PROFILE_MIMETYPE)),
@@ -180,7 +244,7 @@ Qt4ProjectFile::Qt4ProjectFile(Qt4Project *project, const QString &filePath, QOb
 {
 }
 
-bool Qt4ProjectFile::save(const QString &)
+bool Qt4ProjectFile::save(QString *, const QString &, bool)
 {
     // This is never used
     return false;
@@ -236,12 +300,15 @@ Core::IFile::ReloadBehavior Qt4ProjectFile::reloadBehavior(ChangeTrigger state,
     return BehaviorSilent;
 }
 
-void Qt4ProjectFile::reload(ReloadFlag flag, ChangeType type)
+bool Qt4ProjectFile::reload(QString *errorString, ReloadFlag flag, ChangeType type)
 {
+    Q_UNUSED(errorString)
     Q_UNUSED(flag)
     Q_UNUSED(type)
+    return true;
 }
 
+} // namespace Internal
 /*!
   \class Qt4Project
 
@@ -252,20 +319,22 @@ Qt4Project::Qt4Project(Qt4Manager *manager, const QString& fileName) :
     m_manager(manager),
     m_rootProjectNode(0),
     m_nodesWatcher(new Internal::Qt4NodesWatcher(this)),
-    m_targetFactory(new Qt4TargetFactory(this)),
     m_fileInfo(new Qt4ProjectFile(this, fileName, this)),
     m_projectFiles(new Qt4ProjectFiles),
     m_proFileOption(0),
     m_asyncUpdateFutureInterface(0),
     m_pendingEvaluateFuturesCount(0),
     m_asyncUpdateState(NoState),
-    m_cancelEvaluate(false)
+    m_cancelEvaluate(false),
+    m_codeModelCanceled(false),
+    m_centralizedFolderWatcher(0)
 {
+    setProjectContext(Core::Context(Qt4ProjectManager::Constants::PROJECT_ID));
+    setProjectLanguage(Core::Context(ProjectExplorer::Constants::LANG_CXX));
+
     m_asyncUpdateTimer.setSingleShot(true);
     m_asyncUpdateTimer.setInterval(3000);
     connect(&m_asyncUpdateTimer, SIGNAL(timeout()), this, SLOT(asyncUpdate()));
-
-    setSupportedTargetIds(QtVersionManager::instance()->supportedTargetIds());
 }
 
 Qt4Project::~Qt4Project()
@@ -275,7 +344,11 @@ Qt4Project::~Qt4Project()
     m_manager->unregisterProject(this);
     delete m_projectFiles;
     m_cancelEvaluate = true;
-    delete m_rootProjectNode;
+    // Deleting the root node triggers a few things, make sure rootProjectNode
+    // returns 0 already
+    Qt4ProFileNode *root = m_rootProjectNode;
+    m_rootProjectNode = 0;
+    delete root;
 }
 
 void Qt4Project::updateFileList()
@@ -297,11 +370,12 @@ bool Qt4Project::fromMap(const QVariantMap &map)
 
     // Prune targets without buildconfigurations:
     // This can happen esp. when updating from a old version of Qt Creator
-    QList<Target *>ts(targets());
+    QList<Target *>ts = targets();
     foreach (Target *t, ts) {
         if (t->buildConfigurations().isEmpty()) {
             qWarning() << "Removing" << t->id() << "since it has no buildconfigurations!";
             removeTarget(t);
+            delete t;
         }
     }
 
@@ -327,51 +401,48 @@ bool Qt4Project::fromMap(const QVariantMap &map)
     // This might be incorrect, need a full update
     updateCodeModels();
 
-    createApplicationProjects();
+    foreach (Target *t, targets())
+        static_cast<Qt4BaseTarget *>(t)->createApplicationProFiles();
 
     foreach (Target *t, targets())
         onAddedTarget(t);
 
-    setSupportedTargetIds(QtVersionManager::instance()->supportedTargetIds());
+    connect(m_nodesWatcher, SIGNAL(proFileUpdated(Qt4ProjectManager::Qt4ProFileNode*,bool,bool)),
+            this, SIGNAL(proFileUpdated(Qt4ProjectManager::Qt4ProFileNode *,bool,bool)));
+
+    // Now we emit update once :)
+    m_rootProjectNode->emitProFileUpdatedRecursive();
+
 
     // Setup Qt versions supported (== possible targets).
     connect(this, SIGNAL(addedTarget(ProjectExplorer::Target*)),
             this, SLOT(onAddedTarget(ProjectExplorer::Target*)));
 
-    connect(QtVersionManager::instance(), SIGNAL(qtVersionsChanged(QList<int>)),
-            this, SLOT(qtVersionsChanged()));
-
-    connect(m_nodesWatcher, SIGNAL(proFileUpdated(Qt4ProjectManager::Internal::Qt4ProFileNode *)),
-            this, SIGNAL(proFileUpdated(Qt4ProjectManager::Internal::Qt4ProFileNode *)));
-
     connect(this, SIGNAL(activeTargetChanged(ProjectExplorer::Target*)),
             this, SLOT(activeTargetWasChanged()));
 
+    emit fromMapFinished();
+
     return true;
 }
 
-Qt4TargetFactory *Qt4Project::targetFactory() const
+Qt4BaseTarget *Qt4Project::activeTarget() const
 {
-    return m_targetFactory;
-}
-
-Qt4Target *Qt4Project::activeTarget() const
-{
-    return static_cast<Qt4Target *>(Project::activeTarget());
+    return static_cast<Qt4BaseTarget *>(Project::activeTarget());
 }
 
 void Qt4Project::onAddedTarget(ProjectExplorer::Target *t)
 {
     Q_ASSERT(t);
-    Qt4Target *qt4target = qobject_cast<Qt4Target *>(t);
+    Qt4BaseTarget *qt4target = qobject_cast<Qt4BaseTarget *>(t);
     Q_ASSERT(qt4target);
     connect(qt4target, SIGNAL(buildDirectoryInitialized()),
             this, SIGNAL(buildDirectoryInitialized()));
-    connect(qt4target, SIGNAL(proFileEvaluateNeeded(Qt4ProjectManager::Internal::Qt4Target*)),
-            this, SLOT(proFileEvaluateNeeded(Qt4ProjectManager::Internal::Qt4Target*)));
+    connect(qt4target, SIGNAL(proFileEvaluateNeeded(Qt4ProjectManager::Qt4BaseTarget*)),
+            this, SLOT(proFileEvaluateNeeded(Qt4ProjectManager::Qt4BaseTarget*)));
 }
 
-void Qt4Project::proFileEvaluateNeeded(Qt4ProjectManager::Internal::Qt4Target *target)
+void Qt4Project::proFileEvaluateNeeded(Qt4ProjectManager::Qt4BaseTarget *target)
 {
     if (activeTarget() == target)
         scheduleAsyncUpdate();
@@ -416,11 +487,10 @@ void Qt4Project::updateCodeModels()
 
 void Qt4Project::updateCppCodeModel()
 {
-    Qt4BuildConfiguration *activeBC = activeTarget()->activeBuildConfiguration();
+    Qt4BuildConfiguration *activeBC = activeTarget()->activeQt4BuildConfiguration();
 
-    CppTools::CppModelManagerInterface *modelmanager =
-        ExtensionSystem::PluginManager::instance()
-            ->getObject<CppTools::CppModelManagerInterface>();
+    CPlusPlus::CppModelManagerInterface *modelmanager =
+        CPlusPlus::CppModelManagerInterface::instance();
 
     if (!modelmanager)
         return;
@@ -430,21 +500,20 @@ void Qt4Project::updateCppCodeModel()
     QStringList predefinedFrameworkPaths;
     QByteArray predefinedMacros;
 
-    QString qtFrameworkPath = activeBC->qtVersion()->frameworkInstallPath();
+    QString qtFrameworkPath;
+    if (activeBC->qtVersion())
+        qtFrameworkPath = activeBC->qtVersion()->frameworkInstallPath();
     if (!qtFrameworkPath.isEmpty())
         predefinedFrameworkPaths.append(qtFrameworkPath);
 
     ToolChain *tc = activeBC->toolChain();
     if (tc) {
         predefinedMacros = tc->predefinedMacros();
-        //qDebug()<<"Predefined Macros";
-        //qDebug()<<tc->predefinedMacros();
-        //qDebug()<<"";
-        //qDebug()<<"System Header Paths";
-        //foreach(const HeaderPath &hp, tc->systemHeaderPaths())
-        //    qDebug()<<hp.path();
 
-        foreach (const HeaderPath &headerPath, tc->systemHeaderPaths()) {
+        QList<HeaderPath> headers = tc->systemHeaderPaths();
+        if (activeBC->qtVersion())
+            headers.append(activeBC->qtVersion()->systemHeaderPathes());
+        foreach (const HeaderPath &headerPath, headers) {
             if (headerPath.kind() == HeaderPath::FrameworkHeaderPath)
                 predefinedFrameworkPaths.append(headerPath.path());
             else
@@ -461,25 +530,16 @@ void Qt4Project::updateCppCodeModel()
 
     // Collect per .pro file information
     foreach (Qt4ProFileNode *pro, proFiles) {
-        Internal::CodeModelInfo info;
-        info.defines = predefinedMacros;
-        info.frameworkPaths = predefinedFrameworkPaths;
-
-        info.precompiledHeader = pro->variableValue(PrecompiledHeaderVar);
-
-        allPrecompileHeaders.append(info.precompiledHeader);
+        allPrecompileHeaders.append(pro->variableValue(PrecompiledHeaderVar));
 
         // Add custom defines
 
         foreach (const QString &def, pro->variableValue(DefinesVar)) {
             allDefinedMacros += "#define ";
-            info.defines += "#define ";
             const int index = def.indexOf(QLatin1Char('='));
             if (index == -1) {
                 allDefinedMacros += def.toLatin1();
                 allDefinedMacros += " 1\n";
-                info.defines += def.toLatin1();
-                info.defines += " 1\n";
             } else {
                 const QString name = def.left(index);
                 const QString value = def.mid(index + 1);
@@ -487,10 +547,6 @@ void Qt4Project::updateCppCodeModel()
                 allDefinedMacros += ' ';
                 allDefinedMacros += value.toLocal8Bit();
                 allDefinedMacros += '\n';
-                info.defines += name.toLatin1();
-                info.defines += ' ';
-                info.defines += value.toLocal8Bit();
-                info.defines += '\n';
             }
         }
 
@@ -498,40 +554,14 @@ void Qt4Project::updateCppCodeModel()
         foreach (const QString &includePath, proIncludePaths) {
             if (!allIncludePaths.contains(includePath))
                 allIncludePaths.append(includePath);
-            if (!info.includes.contains(includePath))
-                info.includes.append(includePath);
         }
-
-#if 0 // Experimental PKGCONFIG support
-        { // Pkg Config support
-            QStringList pkgConfig = pro->variableValue(PkgConfigVar);
-            if (!pkgConfig.isEmpty()) {
-                pkgConfig.prepend("--cflags-only-I");
-                QProcess process;
-                process.start("pkg-config", pkgConfig);
-                process.waitForFinished();
-                QString result = process.readAllStandardOutput();
-                foreach(const QString &part, result.trimmed().split(' ', QString::SkipEmptyParts)) {
-                    info.includes.append(part.mid(2)); // Chop off "-I"
-                }
-            }
-        }
-#endif
-
-        // Add mkspec directory
-        info.includes.append(activeBC->qtVersion()->mkspecPath());
-        info.includes.append(predefinedIncludePaths);
-
-//        qDebug()<<"Dumping code model information";
-//        qDebug()<<"for .pro file"<< pro->path();
-//        qDebug()<<info.defines;
-//        qDebug()<<info.includes;
-//        qDebug()<<info.frameworkPaths;
-//        qDebug()<<"\n";
     }
 
     // Add mkspec directory
-    allIncludePaths.append(activeBC->qtVersion()->mkspecPath());
+    if (rootQt4ProjectNode())
+        allIncludePaths.append(rootQt4ProjectNode()->resolvedMkspecPath());
+    else if (activeBC->qtVersion())
+        allIncludePaths.append(activeBC->qtVersion()->mkspecPath());
 
     allIncludePaths.append(predefinedIncludePaths);
 
@@ -541,7 +571,7 @@ void Qt4Project::updateCppCodeModel()
     files += m_projectFiles->files[SourceType];
     files += m_projectFiles->generatedFiles[SourceType];
 
-    CppTools::CppModelManagerInterface::ProjectInfo pinfo = modelmanager->projectInfo(this);
+    CPlusPlus::CppModelManagerInterface::ProjectInfo pinfo = modelmanager->projectInfo(this);
 
     //qDebug()<<"Using precompiled header"<<allPrecompileHeaders;
 
@@ -551,7 +581,8 @@ void Qt4Project::updateCppCodeModel()
         && pinfo.includePaths == allIncludePaths
         && pinfo.frameworkPaths == allFrameworkPaths
         && fileList
-        && pinfo.precompiledHeaders == allPrecompileHeaders) {
+        && pinfo.precompiledHeaders == allPrecompileHeaders
+        && !m_codeModelCanceled) {
         // Nothing to update...
     } else {
         pinfo.sourceFiles.clear();
@@ -572,46 +603,55 @@ void Qt4Project::updateCppCodeModel()
 
         modelmanager->updateProjectInfo(pinfo);
         m_codeModelFuture = modelmanager->updateSourceFiles(pinfo.sourceFiles);
+        m_codeModelCanceled = false;
     }
 }
 
 void Qt4Project::updateQmlJSCodeModel()
 {
-    if (m_projectFiles->files[QMLType].isEmpty())
-        return;
-
     QmlJS::ModelManagerInterface *modelManager = QmlJS::ModelManagerInterface::instance();
     if (!modelManager)
         return;
 
     QmlJS::ModelManagerInterface::ProjectInfo projectInfo = modelManager->projectInfo(this);
-
-    // Not essential since the QmlJS engine parses required files on demand.
-    //projectInfo.sourceFiles = ...
+    projectInfo.sourceFiles = m_projectFiles->files[QMLType];
 
     FindQt4ProFiles findQt4ProFiles;
     QList<Qt4ProFileNode *> proFiles = findQt4ProFiles(rootProjectNode());
 
+    projectInfo.importPaths.clear();
     foreach (Qt4ProFileNode *node, proFiles) {
         projectInfo.importPaths.append(node->variableValue(QmlImportPathVar));
     }
+
+    bool preferDebugDump = false;
+    projectInfo.tryQmlDump = false;
     if (activeTarget() && activeTarget()->activeBuildConfiguration()) {
-        const QtVersion *qtVersion = activeTarget()->activeBuildConfiguration()->qtVersion();
-        if (qtVersion->isValid()) {
-            const QString qtVersionImportPath = qtVersion->versionInfo().value("QT_INSTALL_IMPORTS");
-            if (!qtVersionImportPath.isEmpty())
-                projectInfo.importPaths += qtVersionImportPath;
+        preferDebugDump = activeTarget()->activeQt4BuildConfiguration()->qmakeBuildConfiguration() & QtSupport::BaseQtVersion::DebugBuild;
+        QtSupport::BaseQtVersion *qtVersion = activeTarget()->activeQt4BuildConfiguration()->qtVersion();
+        if (qtVersion && qtVersion->isValid()) {
+            projectInfo.tryQmlDump = qtVersion->type() == QtSupport::Constants::DESKTOPQT
+                    || qtVersion->type() == QtSupport::Constants::SIMULATORQT;
+            projectInfo.qtImportsPath = qtVersion->versionInfo().value("QT_INSTALL_IMPORTS");
+            if (!projectInfo.qtImportsPath.isEmpty())
+                projectInfo.importPaths += projectInfo.qtImportsPath;
+            projectInfo.qtVersionString = qtVersion->qtVersionString();
         }
     }
-    QmlDumpTool::pathAndEnvironment(this, &projectInfo.qmlDumpPath, &projectInfo.qmlDumpEnvironment);
     projectInfo.importPaths.removeDuplicates();
 
-    modelManager->updateProjectInfo(projectInfo);
-}
+    if (projectInfo.tryQmlDump) {
+        const Qt4BuildConfiguration *bc = activeTarget()->activeQt4BuildConfiguration();
+        if (bc) {
+            QtSupport::QmlDumpTool::pathAndEnvironment(this, bc->qtVersion(), bc->toolChain(),
+                                                       preferDebugDump, &projectInfo.qmlDumpPath, &projectInfo.qmlDumpEnvironment);
+        }
+    } else {
+        projectInfo.qmlDumpPath.clear();
+        projectInfo.qmlDumpEnvironment.clear();
+    }
 
-void Qt4Project::qtVersionsChanged()
-{
-    setSupportedTargetIds(QtVersionManager::instance()->supportedTargetIds());
+    modelManager->updateProjectInfo(projectInfo);
 }
 
 ///*!
@@ -622,15 +662,21 @@ void Qt4Project::update()
     if (debug)
         qDebug()<<"Doing sync update";
     m_rootProjectNode->update();
+
     if (debug)
         qDebug()<<"State is now Base";
     m_asyncUpdateState = Base;
+    activeTarget()->activeQt4BuildConfiguration()->setEnabled(true);
+    emit proParsingDone();
 }
 
 void Qt4Project::scheduleAsyncUpdate(Qt4ProFileNode *node)
 {
+    if (m_asyncUpdateState == ShuttingDown)
+        return;
+
     if (debug)
-        qDebug()<<"schduleAsyncUpdate (node)";
+        qDebug()<<"schduleAsyncUpdate (node)"<<node->path();
     Q_ASSERT(m_asyncUpdateState != NoState);
 
     if (m_cancelEvaluate) {
@@ -642,6 +688,8 @@ void Qt4Project::scheduleAsyncUpdate(Qt4ProFileNode *node)
         return;
     }
 
+    activeTarget()->activeQt4BuildConfiguration()->setEnabled(false);
+
     if (m_asyncUpdateState == AsyncFullUpdatePending) {
         // Just postpone
         if (debug)
@@ -654,7 +702,7 @@ void Qt4Project::scheduleAsyncUpdate(Qt4ProFileNode *node)
         // Add the node
         m_asyncUpdateState = AsyncPartialUpdatePending;
 
-        QList<Internal::Qt4ProFileNode *>::iterator it;
+        QList<Qt4ProFileNode *>::iterator it;
         bool add = true;
         if (debug)
             qDebug()<<"scheduleAsyncUpdate();"<<m_partialEvaluate.size()<<"nodes";
@@ -664,10 +712,10 @@ void Qt4Project::scheduleAsyncUpdate(Qt4ProFileNode *node)
                 add = false;
                 break;
             } else if (node->isParent(*it)) { // We already have the parent in the list, nothing to do
+                it = m_partialEvaluate.erase(it);
+            } else if ((*it)->isParent(node)) { // The node is the parent of a child already in the list
                 add = false;
                 break;
-            } else if ((*it)->isParent(node)) { // The node is the parent of a child already in the list
-                it = m_partialEvaluate.erase(it);
             } else {
                 ++it;
             }
@@ -695,6 +743,9 @@ void Qt4Project::scheduleAsyncUpdate()
 {
     if (debug)
         qDebug()<<"scheduleAsyncUpdate";
+    if (m_asyncUpdateState == ShuttingDown)
+        return;
+
     Q_ASSERT(m_asyncUpdateState != NoState);
     if (m_cancelEvaluate) { // we are in progress of canceling
                             // and will start the evaluation after that
@@ -707,17 +758,22 @@ void Qt4Project::scheduleAsyncUpdate()
             qDebug()<<"  update in progress, canceling and setting state to full update pending";
         m_cancelEvaluate = true;
         m_asyncUpdateState = AsyncFullUpdatePending;
+        activeTarget()->activeQt4BuildConfiguration()->setEnabled(false);
+        m_rootProjectNode->setParseInProgressRecursive(true);
         return;
     }
 
     if (debug)
         qDebug()<<"  starting timer for full update, setting state to full update pending";
     m_partialEvaluate.clear();
+    activeTarget()->activeQt4BuildConfiguration()->setEnabled(false);
+    m_rootProjectNode->setParseInProgressRecursive(true);
     m_asyncUpdateState = AsyncFullUpdatePending;
     m_asyncUpdateTimer.start();
 
     // Cancel running code model update
     m_codeModelFuture.cancel();
+    m_codeModelCanceled = true;
 }
 
 
@@ -756,12 +812,16 @@ void Qt4Project::decrementPendingEvaluateFutures()
                 qDebug()<<"  Oh update is pending start the timer";
             m_asyncUpdateTimer.start();
         } else  if (m_asyncUpdateState != ShuttingDown){
-            // After beeing done, we need to call:
+            // After being done, we need to call:
+            m_asyncUpdateState = Base;
+            activeTarget()->activeQt4BuildConfiguration()->setEnabled(true);
+            foreach (Target *t, targets())
+                static_cast<Qt4BaseTarget *>(t)->createApplicationProFiles();
             updateFileList();
             updateCodeModels();
+            emit proParsingDone();
             if (debug)
                 qDebug()<<"  Setting state to Base";
-            m_asyncUpdateState = Base;
         }
     }
 }
@@ -771,19 +831,6 @@ bool Qt4Project::wasEvaluateCanceled()
     return m_cancelEvaluate;
 }
 
-QString Qt4Project::defaultTopLevelBuildDirectory() const
-{
-    return defaultTopLevelBuildDirectory(file()->fileName());
-}
-
-QString Qt4Project::defaultTopLevelBuildDirectory(const QString &profilePath)
-{
-    if (profilePath.isEmpty())
-        return QString();
-    QFileInfo info(profilePath);
-    return QDir(projectDirectory(profilePath) + QLatin1String("/../") + info.baseName() + QLatin1String("-build")).absolutePath();
-}
-
 void Qt4Project::asyncUpdate()
 {
     if (debug)
@@ -903,56 +950,96 @@ void Qt4Project::addDefaultBuild()
 
 void Qt4Project::proFileParseError(const QString &errorMessage)
 {
-    Core::ICore::instance()->messageManager()->printToOutputPane(errorMessage);
+    Core::ICore::instance()->messageManager()->printToOutputPanePopup(errorMessage);
 }
 
-ProFileReader *Qt4Project::createProFileReader(Qt4ProFileNode *qt4ProFileNode)
+QtSupport::ProFileReader *Qt4Project::createProFileReader(Qt4ProFileNode *qt4ProFileNode, Qt4BuildConfiguration *bc)
 {
     if (!m_proFileOption) {
         m_proFileOption = new ProFileOption;
         m_proFileOptionRefCnt = 0;
 
-        if (activeTarget() &&
-            activeTarget()->activeBuildConfiguration()) {
-            QtVersion *version = activeTarget()->activeBuildConfiguration()->qtVersion();
-            if (version->isValid())
+        if (!bc && activeTarget())
+            bc = activeTarget()->activeQt4BuildConfiguration();
+
+        if (bc) {
+            QtSupport::BaseQtVersion *version = bc->qtVersion();
+            if (version && version->isValid()) {
                 m_proFileOption->properties = version->versionInfo();
+                if (bc->toolChain())
+                    m_proFileOption->sysroot = bc->qtVersion()->systemRoot();
+            }
+
+            Utils::Environment env = bc->environment();
+            Utils::Environment::const_iterator eit = env.constBegin(), eend = env.constEnd();
+            for (; eit != eend; ++eit)
+                m_proFileOption->environment.insert(env.key(eit), env.value(eit));
+
+            QStringList args;
+            if (QMakeStep *qs = bc->qmakeStep()) {
+                args = qs->parserArguments();
+                m_proFileOption->qmakespec = qs->mkspec();
+            } else {
+                args = bc->configCommandLineArguments();
+            }
+            m_proFileOption->setCommandLineArguments(args);
         }
 
-        ProFileCacheManager::instance()->incRefCount();
+        QtSupport::ProFileCacheManager::instance()->incRefCount();
     }
     ++m_proFileOptionRefCnt;
 
-    ProFileReader *reader = new ProFileReader(m_proFileOption);
+    QtSupport::ProFileReader *reader = new QtSupport::ProFileReader(m_proFileOption);
 
     reader->setOutputDir(qt4ProFileNode->buildDir());
 
     return reader;
 }
 
-void Qt4Project::destroyProFileReader(ProFileReader *reader)
+ProFileOption *Qt4Project::proFileOption()
+{
+    return m_proFileOption;
+}
+
+void Qt4Project::destroyProFileReader(QtSupport::ProFileReader *reader)
 {
     delete reader;
     if (!--m_proFileOptionRefCnt) {
         QString dir = QFileInfo(m_fileInfo->fileName()).absolutePath();
         if (!dir.endsWith(QLatin1Char('/')))
             dir += QLatin1Char('/');
-        ProFileCacheManager::instance()->discardFiles(dir);
-        ProFileCacheManager::instance()->decRefCount();
+        QtSupport::ProFileCacheManager::instance()->discardFiles(dir);
+        QtSupport::ProFileCacheManager::instance()->decRefCount();
 
         delete m_proFileOption;
         m_proFileOption = 0;
     }
 }
 
-Qt4ProFileNode *Qt4Project::rootProjectNode() const
+ProjectExplorer::ProjectNode *Qt4Project::rootProjectNode() const
 {
     return m_rootProjectNode;
 }
 
-BuildConfigWidget *Qt4Project::createConfigWidget()
+Qt4ProFileNode *Qt4Project::rootQt4ProjectNode() const
 {
-    return new Qt4ProjectConfigWidget(this);
+    return m_rootProjectNode;
+}
+
+bool Qt4Project::validParse(const QString &proFilePath) const
+{
+    if (!m_rootProjectNode)
+        return false;
+    const Qt4ProFileNode *node = m_rootProjectNode->findProFileFor(proFilePath);
+    return node && node->validParse();
+}
+
+bool Qt4Project::parseInProgress(const QString &proFilePath) const
+{
+    if (!m_rootProjectNode)
+        return false;
+    const Qt4ProFileNode *node = m_rootProjectNode->findProFileFor(proFilePath);
+    return node && node->parseInProgress();
 }
 
 QList<BuildConfigWidget*> Qt4Project::subConfigWidgets()
@@ -962,23 +1049,21 @@ QList<BuildConfigWidget*> Qt4Project::subConfigWidgets()
     return subWidgets;
 }
 
-void Qt4Project::collectLeafProFiles(QList<Qt4ProFileNode *> &list, Qt4ProFileNode *node)
+void Qt4Project::collectAllfProFiles(QList<Qt4ProFileNode *> &list, Qt4ProFileNode *node)
 {
-    if (node->projectType() != Internal::SubDirsTemplate) {
-        list.append(node);
-    }
+    list.append(node);
     foreach (ProjectNode *n, node->subProjectNodes()) {
         Qt4ProFileNode *qt4ProFileNode = qobject_cast<Qt4ProFileNode *>(n);
         if (qt4ProFileNode)
-            collectLeafProFiles(list, qt4ProFileNode);
+            collectAllfProFiles(list, qt4ProFileNode);
     }
 }
 
 
 void Qt4Project::collectApplicationProFiles(QList<Qt4ProFileNode *> &list, Qt4ProFileNode *node)
 {
-    if (node->projectType() == Internal::ApplicationTemplate
-        || node->projectType() == Internal::ScriptTemplate) {
+    if (node->projectType() == ApplicationTemplate
+        || node->projectType() == ScriptTemplate) {
         list.append(node);
     }
     foreach (ProjectNode *n, node->subProjectNodes()) {
@@ -988,57 +1073,12 @@ void Qt4Project::collectApplicationProFiles(QList<Qt4ProFileNode *> &list, Qt4Pr
     }
 }
 
-void Qt4Project::createApplicationProjects()
-{
-    foreach (Target *target, targets()) {
-        if (target->runConfigurations().count()) {
-            // Remove all run configurations which the new project wizard created
-            QList<RunConfiguration*> toRemove;
-            foreach (RunConfiguration * rc, target->runConfigurations()) {
-                CustomExecutableRunConfiguration *cerc = qobject_cast<CustomExecutableRunConfiguration *>(rc);
-                if (cerc && !cerc->isConfigured())
-                    toRemove.append(rc);
-            }
-            foreach (RunConfiguration *rc, toRemove)
-                target->removeRunConfiguration(rc);
-        }
-
-        // We use the list twice
-        QList<Qt4ProFileNode *> profiles = applicationProFiles();
-        QStringList paths;
-        foreach (Qt4ProFileNode *pro, profiles)
-            paths << pro->path();
-
-        foreach (RunConfiguration *rc, target->runConfigurations()) {
-            if (Qt4RunConfiguration *qt4rc = qobject_cast<Qt4RunConfiguration *>(rc)) {
-                if (!paths.contains(qt4rc->proFilePath())) {
-                    // A deleted .pro file? or a change template
-                    // We do remove those though
-                    target->removeRunConfiguration(rc);
-                }
-            }
-        }
-
-        // Only add new runconfigurations if there are none.
-        if (target->runConfigurations().isEmpty()) {
-            Qt4Target *qt4Target = static_cast<Qt4Target *>(target);
-            foreach (Qt4ProFileNode *qt4proFile, profiles) {
-                qt4Target->addRunConfigurationForPath(qt4proFile->path());
-            }
-        }
-        // Oh still none? Add a custom executable runconfiguration
-        if (target->runConfigurations().isEmpty()) {
-            target->addRunConfiguration(new ProjectExplorer::CustomExecutableRunConfiguration(target));
-        }
-    }
-}
-
-QList<Qt4ProFileNode *> Qt4Project::leafProFiles() const
+QList<Qt4ProFileNode *> Qt4Project::allProFiles() const
 {
     QList<Qt4ProFileNode *> list;
     if (!rootProjectNode())
         return list;
-    collectLeafProFiles(list, rootProjectNode());
+    collectAllfProFiles(list, rootQt4ProjectNode());
     return list;
 }
 
@@ -1047,7 +1087,7 @@ QList<Qt4ProFileNode *> Qt4Project::applicationProFiles() const
     QList<Qt4ProFileNode *> list;
     if (!rootProjectNode())
         return list;
-    collectApplicationProFiles(list, rootProjectNode());
+    collectApplicationProFiles(list, rootQt4ProjectNode());
     return list;
 }
 
@@ -1108,17 +1148,28 @@ void Qt4Project::notifyChanged(const QString &name)
 {
     if (files(Qt4Project::ExcludeGeneratedFiles).contains(name)) {
         QList<Qt4ProFileNode *> list;
-        findProFile(name, rootProjectNode(), list);
+        findProFile(name, rootQt4ProjectNode(), list);
         foreach(Qt4ProFileNode *node, list) {
-            ProFileCacheManager::instance()->discardFile(name);
+            QtSupport::ProFileCacheManager::instance()->discardFile(name);
             node->update();
         }
+        updateFileList();
     }
 }
 
-CentralizedFolderWatcher *Qt4Project::centralizedFolderWatcher()
+void Qt4Project::watchFolders(const QStringList &l, Qt4PriFileNode *node)
 {
-    return &m_centralizedFolderWatcher;
+    if (l.isEmpty())
+        return;
+    if (!m_centralizedFolderWatcher)
+        m_centralizedFolderWatcher = new Internal::CentralizedFolderWatcher(this);
+    m_centralizedFolderWatcher->watchFolders(l, node);
+}
+
+void Qt4Project::unwatchFolders(const QStringList &l, Qt4PriFileNode *node)
+{
+    if (m_centralizedFolderWatcher && !l.isEmpty())
+        m_centralizedFolderWatcher->unwatchFolders(l, node);
 }
 
 /////////////
@@ -1131,7 +1182,7 @@ namespace {
    bool debugCFW = false;
 }
 
-CentralizedFolderWatcher::CentralizedFolderWatcher()
+CentralizedFolderWatcher::CentralizedFolderWatcher(QObject *parent) : QObject(parent)
 {
     m_compressTimer.setSingleShot(true);
     m_compressTimer.setInterval(200);
@@ -1152,13 +1203,14 @@ QSet<QString> CentralizedFolderWatcher::recursiveDirs(const QString &folder)
     QDir dir(folder);
     QStringList list = dir.entryList(QDir::Dirs | QDir::NoSymLinks | QDir::NoDotAndDotDot);
     foreach (const QString &f, list) {
-        result.insert(folder + f + "/");
-        result += recursiveDirs(folder + f + "/");
+        const QString a = folder + f + '/';
+        result.insert(a);
+        result += recursiveDirs(a);
     }
     return result;
 }
 
-void CentralizedFolderWatcher::watchFolders(const QList<QString> &folders, Qt4PriFileNode *node)
+void CentralizedFolderWatcher::watchFolders(const QList<QString> &folders, Qt4ProjectManager::Qt4PriFileNode *node)
 {
     if (debugCFW)
         qDebug()<<"CFW::watchFolders()"<<folders<<"for node"<<node->path();
@@ -1182,7 +1234,7 @@ void CentralizedFolderWatcher::watchFolders(const QList<QString> &folders, Qt4Pr
     }
 }
 
-void CentralizedFolderWatcher::unwatchFolders(const QList<QString> &folders, Qt4PriFileNode *node)
+void CentralizedFolderWatcher::unwatchFolders(const QList<QString> &folders, Qt4ProjectManager::Qt4PriFileNode *node)
 {
     if (debugCFW)
         qDebug()<<"CFW::unwatchFolders()"<<folders<<"for node"<<node->path();
@@ -1206,7 +1258,7 @@ void CentralizedFolderWatcher::unwatchFolders(const QList<QString> &folders, Qt4
                 // So the rwf is a subdirectory of a folder we aren't watching
                 // but maybe someone else wants us to watch
                 bool needToWatch = false;
-                QMultiMap<QString, Qt4PriFileNode *>::const_iterator it, end;
+                QMultiMap<QString, Qt4ProjectManager::Qt4PriFileNode *>::const_iterator it, end;
                 end = m_map.constEnd();
                 for (it = m_map.constEnd(); it != end; ++it) {
                     if (rwf.startsWith(it.key())) {
@@ -1253,8 +1305,8 @@ void CentralizedFolderWatcher::delayedFolderChanged(const QString &folder)
     while (true) {
         if (!dir.endsWith('/'))
             dir.append('/');
-        QList<Qt4PriFileNode *> nodes = m_map.values(dir);
-        foreach (Qt4PriFileNode *node, nodes) {
+        QList<Qt4ProjectManager::Qt4PriFileNode *> nodes = m_map.values(dir);
+        foreach (Qt4ProjectManager::Qt4PriFileNode *node, nodes) {
             node->folderChanged(folder);
         }
 
@@ -1316,4 +1368,6 @@ void CentralizedFolderWatcher::delayedFolderChanged(const QString &folder)
 //    }
 //}
 
+} // namespace Qt4ProjectManager
 
+#include "qt4project.moc"
