@@ -6,26 +6,25 @@
 **
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
-** No Commercial Usage
-**
-** This file contains pre-release code and may not be distributed.
-** You may use this file in accordance with the terms and conditions
-** contained in the Technology Preview License Agreement accompanying
-** this package.
 **
 ** GNU Lesser General Public License Usage
 **
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this file.
+** Please review the following information to ensure the GNU Lesser General
+** Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
+** Other Usage
+**
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
 ** If you have questions regarding the use of this file, please contact
 ** Nokia at qt-info@nokia.com.
 **
@@ -45,8 +44,10 @@
 #include <vcsbase/vcsbaseeditor.h>
 #include <vcsbase/basevcssubmiteditorfactory.h>
 #include <vcsbase/vcsbaseoutputwindow.h>
+#include <vcsbase/vcsbaseeditorparameterwidget.h>
 #include <utils/synchronousprocess.h>
 #include <utils/parameteraction.h>
+#include <utils/fileutils.h>
 
 #include <coreplugin/icore.h>
 #include <coreplugin/coreconstants.h>
@@ -56,6 +57,7 @@
 #include <coreplugin/actionmanager/actionmanager.h>
 #include <coreplugin/actionmanager/actioncontainer.h>
 #include <coreplugin/actionmanager/command.h>
+#include <coreplugin/id.h>
 #include <coreplugin/editormanager/editormanager.h>
 
 #include <locator/commandlocator.h>
@@ -79,7 +81,8 @@
 #include <QtXml/QXmlStreamReader>
 #include <limits.h>
 
-using namespace Subversion::Internal;
+namespace Subversion {
+namespace Internal {
 
 static const char * const CMD_ID_SUBVERSION_MENU    = "Subversion.Menu";
 static const char * const CMD_ID_ADD                = "Subversion.Add";
@@ -142,7 +145,7 @@ static const VCSBase::VCSBaseEditorParameters editorParameters[] = {
 static inline const VCSBase::VCSBaseEditorParameters *findType(int ie)
 {
     const VCSBase::EditorContentType et = static_cast<VCSBase::EditorContentType>(ie);
-    return  VCSBase::VCSBaseEditor::findType(editorParameters, sizeof(editorParameters)/sizeof(VCSBase::VCSBaseEditorParameters), et);
+    return  VCSBase::VCSBaseEditorWidget::findType(editorParameters, sizeof(editorParameters)/sizeof(VCSBase::VCSBaseEditorParameters), et);
 }
 
 static inline QString debugCodec(const QTextCodec *c)
@@ -150,14 +153,6 @@ static inline QString debugCodec(const QTextCodec *c)
     return c ? QString::fromAscii(c->name()) : QString::fromAscii("Null codec");
 }
 
-Core::IEditor* locateEditor(const char *property, const QString &entry)
-{
-    foreach (Core::IEditor *ed, Core::EditorManager::instance()->openedEditors())
-        if (ed->property(property).toString() == entry)
-            return ed;
-    return 0;
-}
-
 // Parse "svn status" output for added/modified/deleted files
 // "M<7blanks>file"
 typedef QList<SubversionSubmitEditor::StatusFilePair> StatusList;
@@ -272,7 +267,7 @@ bool SubversionPlugin::initialize(const QStringList & /*arguments */, QString *e
     using namespace Core::Constants;
     using namespace ExtensionSystem;
 
-    VCSBase::VCSBasePlugin::initialize(new SubversionControl(this));
+    initializeVcs(new SubversionControl(this));
 
     m_subversionPluginInstance = this;
     Core::ICore *core = Core::ICore::instance();
@@ -466,7 +461,7 @@ bool SubversionPlugin::initialize(const QStringList & /*arguments */, QString *e
     command->setAttribute(Core::Command::CA_UpdateText);
     connect(m_submitCurrentLogAction, SIGNAL(triggered()), this, SLOT(submitCurrentLog()));
 
-    m_submitDiffAction = new QAction(VCSBase::VCSBaseSubmitEditor::diffIcon(), tr("Diff Selected Files"), this);
+    m_submitDiffAction = new QAction(VCSBase::VCSBaseSubmitEditor::diffIcon(), tr("Diff &Selected Files"), this);
     command = ami->registerAction(m_submitDiffAction , Constants::DIFF_SELECTED, svncommitcontext);
 
     m_submitUndoAction = new QAction(tr("&Undo"), this);
@@ -518,10 +513,9 @@ bool SubversionPlugin::submitEditorAboutToClose(VCSBase::VCSBaseSubmitEditor *su
     bool closeEditor = true;
     if (!fileList.empty()) {
         // get message & commit
-        Core::ICore::instance()->fileManager()->blockFileChange(fileIFace);
-        fileIFace->save();
-        Core::ICore::instance()->fileManager()->unblockFileChange(fileIFace);
-        closeEditor= commit(m_commitMessageFileName, fileList);
+        closeEditor = Core::ICore::instance()->fileManager()->saveFile(fileIFace);
+        if (closeEditor)
+            closeEditor = commit(m_commitMessageFileName, fileList);
     }
     if (closeEditor)
         cleanCommitMessageFile();
@@ -533,55 +527,120 @@ void SubversionPlugin::diffCommitFiles(const QStringList &files)
     svnDiff(m_commitRepository, files);
 }
 
+// Collect all parameters required for a diff to be able to associate them
+// with a diff editor and re-run the diff with parameters.
+struct SubversionDiffParameters
+{
+    QString workingDir;
+    QStringList arguments;
+    QStringList files;
+    QString diffName;
+};
+
+// Parameter widget controlling whitespace diff mode, associated with a parameter
+class SubversionDiffParameterWidget : public VCSBase::VCSBaseEditorParameterWidget
+{
+    Q_OBJECT
+public:
+    explicit SubversionDiffParameterWidget(const SubversionDiffParameters &p, QWidget *parent = 0);
+
+signals:
+    void reRunDiff(const Subversion::Internal::SubversionDiffParameters &);
+
+private slots:
+    void triggerReRun();
+
+private:
+    const SubversionDiffParameters m_parameters;
+};
+
+SubversionDiffParameterWidget::SubversionDiffParameterWidget(const SubversionDiffParameters &p, QWidget *parent) :
+    VCSBase::VCSBaseEditorParameterWidget(parent), m_parameters(p)
+{
+    setBaseArguments(p.arguments);
+    addToggleButton(QLatin1String("w"), tr("Ignore whitespace"));
+    connect(this, SIGNAL(argumentsChanged()), this, SLOT(triggerReRun()));
+}
+
+void SubversionDiffParameterWidget::triggerReRun()
+{
+    SubversionDiffParameters effectiveParameters = m_parameters;
+    // Subversion wants" -x -<ext-args>", default being -u
+    const QStringList a = arguments();
+    if (!a.isEmpty())
+        effectiveParameters.arguments << QLatin1String("-x") << (QLatin1String("-u") + a.join(QString()));
+    emit reRunDiff(effectiveParameters);
+}
+
 static inline void setDiffBaseDirectory(Core::IEditor *editor, const QString &db)
 {
-    if (VCSBase::VCSBaseEditor *ve = qobject_cast<VCSBase::VCSBaseEditor*>(editor->widget()))
+    if (VCSBase::VCSBaseEditorWidget *ve = qobject_cast<VCSBase::VCSBaseEditorWidget*>(editor->widget()))
         ve->setDiffBaseDirectory(db);
 }
 
 void SubversionPlugin::svnDiff(const QString &workingDir, const QStringList &files, QString diffname)
 {
-    if (Subversion::Constants::debug)
-        qDebug() << Q_FUNC_INFO << files << diffname;
-    const QString source = VCSBase::VCSBaseEditor::getSource(workingDir, files);
-    QTextCodec *codec = source.isEmpty() ? static_cast<QTextCodec *>(0) : VCSBase::VCSBaseEditor::getCodec(source);
+    SubversionDiffParameters p;
+    p.workingDir = workingDir;
+    p.files = files;
+    p.diffName = diffname;
+    svnDiff(p);
+}
 
-    if (files.count() == 1 && diffname.isEmpty())
-        diffname = QFileInfo(files.front()).fileName();
+void SubversionPlugin::svnDiff(const Subversion::Internal::SubversionDiffParameters &p)
+{
+    if (Subversion::Constants::debug)
+        qDebug() << Q_FUNC_INFO << p.files << p.diffName;
+    const QString source = VCSBase::VCSBaseEditorWidget::getSource(p.workingDir, p.files);
+    QTextCodec *codec = source.isEmpty() ? static_cast<QTextCodec *>(0) : VCSBase::VCSBaseEditorWidget::getCodec(source);
+
+    const QString diffName = p.files.count() == 1 && p.diffName.isEmpty() ?
+                             QFileInfo(p.files.front()).fileName() : p.diffName;
 
     QStringList args(QLatin1String("diff"));
-    args << files;
+    args.append(p.arguments);
+    args << p.files;
 
     const SubversionResponse response =
-            runSvn(workingDir, args, m_settings.timeOutMS(), 0);
+            runSvn(p.workingDir, args, m_settings.timeOutMS(), 0, codec);
     if (response.error)
         return;
 
     // diff of a single file? re-use an existing view if possible to support
     // the common usage pattern of continuously changing and diffing a file
-    if (files.count() == 1) {
-        // Show in the same editor if diff has been executed before
-        if (Core::IEditor *editor = locateEditor("originalFileName", files.front())) {
-            editor->createNew(response.stdOut);
-            Core::EditorManager::instance()->activateEditor(editor, Core::EditorManager::ModeSwitch);
-            setDiffBaseDirectory(editor, workingDir);
-            return;
-        }
+    const QString tag = VCSBase::VCSBaseEditorWidget::editorTag(VCSBase::DiffOutput, p.workingDir, p.files);
+    // Show in the same editor if diff has been executed before
+    if (Core::IEditor *existingEditor = VCSBase::VCSBaseEditorWidget::locateEditorByTag(tag)) {
+        existingEditor->createNew(response.stdOut);
+        Core::EditorManager::instance()->activateEditor(existingEditor, Core::EditorManager::ModeSwitch);
+        setDiffBaseDirectory(existingEditor, p.workingDir);
+        return;
     }
-    const QString title = QString::fromLatin1("svn diff %1").arg(diffname);
+    const QString title = QString::fromLatin1("svn diff %1").arg(diffName);
     Core::IEditor *editor = showOutputInEditor(title, response.stdOut, VCSBase::DiffOutput, source, codec);
-    setDiffBaseDirectory(editor, workingDir);
-    if (files.count() == 1)
-        editor->setProperty("originalFileName", files.front());
+    setDiffBaseDirectory(editor, p.workingDir);
+    VCSBase::VCSBaseEditorWidget::tagEditor(editor, tag);
+    SubversionEditor *diffEditorWidget = qobject_cast<SubversionEditor *>(editor->widget());
+    QTC_ASSERT(diffEditorWidget, return ; )
+
+    // Wire up the parameter widget to trigger a re-run on
+    // parameter change and 'revert' from inside the diff editor.
+    diffEditorWidget->setRevertDiffChunkEnabled(true);
+    SubversionDiffParameterWidget *pw = new SubversionDiffParameterWidget(p);
+    connect(pw, SIGNAL(reRunDiff(Subversion::Internal::SubversionDiffParameters)),
+            this, SLOT(svnDiff(Subversion::Internal::SubversionDiffParameters)));
+    connect(diffEditorWidget, SIGNAL(diffChunkReverted(VCSBase::DiffChunk)),
+            pw, SLOT(triggerReRun()));
+    diffEditorWidget->setConfigurationWidget(pw);
 }
 
 SubversionSubmitEditor *SubversionPlugin::openSubversionSubmitEditor(const QString &fileName)
 {
     Core::IEditor *editor = Core::EditorManager::instance()->openEditor(fileName,
-                                                                        QLatin1String(Constants::SUBVERSIONCOMMITEDITOR_ID),
+                                                                        Constants::SUBVERSIONCOMMITEDITOR_ID,
                                                                         Core::EditorManager::ModeSwitch);
     SubversionSubmitEditor *submitEditor = qobject_cast<SubversionSubmitEditor*>(editor);
-    QTC_ASSERT(submitEditor, /**/);
+    QTC_CHECK(submitEditor);
     submitEditor->registerActions(m_submitUndoAction, m_submitRedoAction, m_submitCurrentLogAction, m_submitDiffAction);
     connect(submitEditor, SIGNAL(diffSelectedFiles(QStringList)), this, SLOT(diffCommitFiles(QStringList)));
     submitEditor->setCheckScriptWorkingDirectory(m_commitRepository);
@@ -750,19 +809,17 @@ void SubversionPlugin::startCommit(const QString &workingDir, const QStringList
     }
     m_commitRepository = workingDir;
     // Create a new submit change file containing the submit template
-    QTemporaryFile changeTmpFile;
-    changeTmpFile.setAutoRemove(false);
-    if (!changeTmpFile.open()) {
-        VCSBase::VCSBaseOutputWindow::instance()->appendError(tr("Cannot create temporary file: %1").arg(changeTmpFile.errorString()));
-        return;
-    }
-    m_commitMessageFileName = changeTmpFile.fileName();
-    // TODO: Regitctrieve submit template from
+    Utils::TempFileSaver saver;
+    saver.setAutoRemove(false);
+    // TODO: Retrieve submit template from
     const QString submitTemplate;
     // Create a submit
-    changeTmpFile.write(submitTemplate.toUtf8());
-    changeTmpFile.flush();
-    changeTmpFile.close();
+    saver.write(submitTemplate.toUtf8());
+    if (!saver.finalize()) {
+        VCSBase::VCSBaseOutputWindow::instance()->appendError(saver.errorString());
+        return;
+    }
+    m_commitMessageFileName = saver.fileName();
     // Create a submit editor and set file list
     SubversionSubmitEditor *editor = openSubversionSubmitEditor(m_commitMessageFileName);
     editor->setStatusList(statusOutput);
@@ -845,7 +902,6 @@ void SubversionPlugin::filelog(const QString &workingDir,
                                const QStringList &files,
                                bool enableAnnotationContextMenu)
 {
-    QTextCodec *codec = VCSBase::VCSBaseEditor::getCodec(workingDir, files);
     // no need for temp file
     QStringList args(QLatin1String("log"));
     if (m_settings.logCount > 0)
@@ -853,26 +909,29 @@ void SubversionPlugin::filelog(const QString &workingDir,
     foreach(const QString &file, files)
         args.append(QDir::toNativeSeparators(file));
 
+    // subversion stores log in UTF-8 and returns it back in user system locale.
+    // So we do not need to encode it.
     const SubversionResponse response =
             runSvn(workingDir, args, m_settings.timeOutMS(),
-                   SshPasswordPrompt, codec);
+                   SshPasswordPrompt, 0/*codec*/);
     if (response.error)
         return;
 
     // Re-use an existing view if possible to support
     // the common usage pattern of continuously changing and diffing a file
 
-    const QString id = VCSBase::VCSBaseEditor::getTitleId(workingDir, files);
-    if (Core::IEditor *editor = locateEditor("logFileName", id)) {
+    const QString id = VCSBase::VCSBaseEditorWidget::getTitleId(workingDir, files);
+    const QString tag = VCSBase::VCSBaseEditorWidget::editorTag(VCSBase::LogOutput, workingDir, files);
+    if (Core::IEditor *editor = VCSBase::VCSBaseEditorWidget::locateEditorByTag(tag)) {
         editor->createNew(response.stdOut);
         Core::EditorManager::instance()->activateEditor(editor, Core::EditorManager::ModeSwitch);
     } else {
         const QString title = QString::fromLatin1("svn log %1").arg(id);
-        const QString source = VCSBase::VCSBaseEditor::getSource(workingDir, files);
-        Core::IEditor *newEditor = showOutputInEditor(title, response.stdOut, VCSBase::LogOutput, source, codec);
-        newEditor->setProperty("logFileName", id);
+        const QString source = VCSBase::VCSBaseEditorWidget::getSource(workingDir, files);
+        Core::IEditor *newEditor = showOutputInEditor(title, response.stdOut, VCSBase::LogOutput, source, /*codec*/0);
+        VCSBase::VCSBaseEditorWidget::tagEditor(newEditor, tag);
         if (enableAnnotationContextMenu)
-            VCSBase::VCSBaseEditor::getVcsBaseEditor(newEditor)->setFileLogAnnotateEnabled(true);
+            VCSBase::VCSBaseEditorWidget::getVcsBaseEditor(newEditor)->setFileLogAnnotateEnabled(true);
     }
 }
 
@@ -915,7 +974,8 @@ void SubversionPlugin::vcsAnnotate(const QString &workingDir, const QString &fil
                                 const QString &revision /* = QString() */,
                                 int lineNumber /* = -1 */)
 {
-    QTextCodec *codec = VCSBase::VCSBaseEditor::getCodec(file);
+    const QString source = VCSBase::VCSBaseEditorWidget::getSource(workingDir, file);
+    QTextCodec *codec = VCSBase::VCSBaseEditorWidget::getCodec(source);
 
     QStringList args(QLatin1String("annotate"));
     if (m_settings.spaceIgnorantAnnotation)
@@ -927,27 +987,27 @@ void SubversionPlugin::vcsAnnotate(const QString &workingDir, const QString &fil
 
     const SubversionResponse response =
             runSvn(workingDir, args, m_settings.timeOutMS(),
-                   SshPasswordPrompt, codec);
+                   SshPasswordPrompt|ForceCLocale, codec);
     if (response.error)
         return;
 
     // Re-use an existing view if possible to support
     // the common usage pattern of continuously changing and diffing a file
-    const QString source = workingDir + QLatin1Char('/') + file;
     if (lineNumber <= 0)
-        lineNumber = VCSBase::VCSBaseEditor::lineNumberOfCurrentEditor(source);
+        lineNumber = VCSBase::VCSBaseEditorWidget::lineNumberOfCurrentEditor(source);
     // Determine id
-    const QString id = VCSBase::VCSBaseEditor::getTitleId(workingDir, QStringList(file), revision);
-
-    if (Core::IEditor *editor = locateEditor("annotateFileName", id)) {
+    const QStringList files = QStringList(file);
+    const QString id = VCSBase::VCSBaseEditorWidget::getTitleId(workingDir, files, revision);
+    const QString tag = VCSBase::VCSBaseEditorWidget::editorTag(VCSBase::AnnotateOutput, workingDir, files);
+    if (Core::IEditor *editor = VCSBase::VCSBaseEditorWidget::locateEditorByTag(tag)) {
         editor->createNew(response.stdOut);
-        VCSBase::VCSBaseEditor::gotoLineOfEditor(editor, lineNumber);
+        VCSBase::VCSBaseEditorWidget::gotoLineOfEditor(editor, lineNumber);
         Core::EditorManager::instance()->activateEditor(editor, Core::EditorManager::ModeSwitch);
     } else {
         const QString title = QString::fromLatin1("svn annotate %1").arg(id);
         Core::IEditor *newEditor = showOutputInEditor(title, response.stdOut, VCSBase::AnnotateOutput, source, codec);
-        newEditor->setProperty("annotateFileName", id);
-        VCSBase::VCSBaseEditor::gotoLineOfEditor(newEditor, lineNumber);
+        VCSBase::VCSBaseEditorWidget::tagEditor(newEditor, tag);
+        VCSBase::VCSBaseEditorWidget::gotoLineOfEditor(newEditor, lineNumber);
     }
 }
 
@@ -993,7 +1053,7 @@ void SubversionPlugin::describe(const QString &source, const QString &changeNr)
     QTextStream(&diffArg) << (number - 1) << ':' << number;
     args.push_back(diffArg);
 
-    QTextCodec *codec = VCSBase::VCSBaseEditor::getCodec(source);
+    QTextCodec *codec = VCSBase::VCSBaseEditorWidget::getCodec(source);
     const SubversionResponse response =
             runSvn(topLevel, args, m_settings.timeOutMS(),
                    SshPasswordPrompt, codec);
@@ -1004,13 +1064,14 @@ void SubversionPlugin::describe(const QString &source, const QString &changeNr)
     // Re-use an existing view if possible to support
     // the common usage pattern of continuously changing and diffing a file
     const QString id = diffArg + source;
-    if (Core::IEditor *editor = locateEditor("describeChange", id)) {
+    const QString tag = VCSBase::VCSBaseEditorWidget::editorTag(VCSBase::DiffOutput, source, QStringList(), changeNr);
+    if (Core::IEditor *editor = VCSBase::VCSBaseEditorWidget::locateEditorByTag(tag)) {
         editor->createNew(description);
         Core::EditorManager::instance()->activateEditor(editor, Core::EditorManager::ModeSwitch);
     } else {
         const QString title = QString::fromLatin1("svn describe %1#%2").arg(fi.fileName(), changeNr);
         Core::IEditor *newEditor = showOutputInEditor(title, description, VCSBase::DiffOutput, source, codec);
-        newEditor->setProperty("describeChange", id);
+        VCSBase::VCSBaseEditorWidget::tagEditor(newEditor, tag);
     }
 }
 
@@ -1124,7 +1185,7 @@ Core::IEditor * SubversionPlugin::showOutputInEditor(const QString& title, const
         e->setSource(source);
     if (codec)
         e->setCodec(codec);
-    Core::IEditor *ie = e->editableInterface();
+    Core::IEditor *ie = e->editor();
     Core::EditorManager::instance()->activateEditor(ie, Core::EditorManager::ModeSwitch);
     return ie;
 }
@@ -1140,10 +1201,11 @@ void SubversionPlugin::setSettings(const SubversionSettings &s)
         m_settings = s;
         if (QSettings *settings = Core::ICore::instance()->settings())
             m_settings.toSettings(settings);
+        subVersionControl()->emitConfigurationChanged();
     }
 }
 
-SubversionPlugin *SubversionPlugin::subversionPluginInstance()
+SubversionPlugin *SubversionPlugin::instance()
 {
     QTC_ASSERT(m_subversionPluginInstance, return m_subversionPluginInstance);
     return m_subversionPluginInstance;
@@ -1221,11 +1283,9 @@ bool SubversionPlugin::vcsMove(const QString &workingDir, const QString &from, c
 {
     QStringList args(QLatin1String("move"));
     args << QDir::toNativeSeparators(from) << QDir::toNativeSeparators(to);
-    qDebug()<<args;
     const SubversionResponse response =
             runSvn(workingDir, args, m_settings.timeOutMS(),
-                   SshPasswordPrompt|ShowStdOutInLogWindow);
-    qDebug() << response.stdOut << "\n"<<response.stdErr;
+                   SshPasswordPrompt|ShowStdOutInLogWindow|FullySynchronously);
     return !response.error;
 }
 
@@ -1238,8 +1298,7 @@ bool SubversionPlugin::vcsCheckout(const QString &directory, const QByteArray &u
     QStringList args = QStringList(QLatin1String("checkout"));
     args << QLatin1String(nonInteractiveOptionC) ;
 
-    if(!username.isEmpty() && !password.isEmpty())
-    {
+    if (!username.isEmpty() && !password.isEmpty()) {
         // If url contains username and password we have to use separate username and password
         // arguments instead of passing those in the url. Otherwise the subversion 'non-interactive'
         // authentication will always fail (if the username and password data are not stored locally),
@@ -1298,49 +1357,56 @@ QString SubversionPlugin::vcsGetRepositoryURL(const QString &directory)
     return QString();
 }
 
-/* Subversion has ".svn" directory in each directory
- * it manages. The top level is the first directory
- * under the directory that does not have a  ".svn". */
 bool SubversionPlugin::managesDirectory(const QString &directory, QString *topLevel /* = 0 */) const
 {
     const QDir dir(directory);
+    if (!dir.exists())
+        return false;
+
     if (topLevel)
         topLevel->clear();
-    bool manages = false;
-    do {
-        if (!dir.exists() || !checkSVNSubDir(dir))
-            break;
-        manages = true;
-        if (!topLevel)
-            break;
-        /* Recursing up, the top level is a child of the first directory that does
-         * not have a  ".svn" directory. The starting directory must be a managed
-         * one. Go up and try to find the first unmanaged parent dir. */
-        QDir lastDirectory = dir;
-        for (QDir parentDir = lastDirectory; parentDir.cdUp() ; lastDirectory = parentDir) {
-            if (!checkSVNSubDir(parentDir)) {
-                *topLevel = lastDirectory.absolutePath();
-                break;
+
+    /* Subversion >= 1.7 has ".svn" directory in the root of the working copy. Check for
+     * furthest parent containing ".svn/wc.db". Need to check for furthest parent as closer
+     * parents may be svn:externals. */
+    QDir parentDir = dir;
+    while (parentDir.cdUp()) {
+        if (checkSVNSubDir(parentDir, QLatin1String("wc.db"))) {
+            if (topLevel)
+                *topLevel = parentDir.absolutePath();
+            return true;
+        }
+    }
+
+    /* Subversion < 1.7 has ".svn" directory in each directory
+     * it manages. The top level is the first directory
+     * under the directory that does not have a  ".svn".*/
+    if (!checkSVNSubDir(dir))
+        return false;
+
+     if (topLevel) {
+         QDir lastDirectory = dir;
+         for (parentDir = lastDirectory; parentDir.cdUp() ; lastDirectory = parentDir) {
+             if (!checkSVNSubDir(parentDir)) {
+                 *topLevel = lastDirectory.absolutePath();
+                 break;
             }
         }
-    } while (false);
-    if (Subversion::Constants::debug) {
-        QDebug nsp = qDebug().nospace();
-        nsp << "SubversionPlugin::managesDirectory" << directory << manages;
-        if (topLevel)
-            nsp << *topLevel;
     }
-    return manages;
+    return true;
 }
 
 // Check whether SVN management subdirs exist.
-bool SubversionPlugin::checkSVNSubDir(const QDir &directory) const
+bool SubversionPlugin::checkSVNSubDir(const QDir &directory, const QString &fileName) const
 {
     const int dirCount = m_svnDirectories.size();
     for (int i = 0; i < dirCount; i++) {
         const QString svnDir = directory.absoluteFilePath(m_svnDirectories.at(i));
-        if (QFileInfo(svnDir).isDir())
-            return true;
+        if (!QFileInfo(svnDir).isDir())
+            continue;
+        if (!fileName.isEmpty() && !QDir(svnDir).exists(fileName))
+            continue;
+        return true;
     }
     return false;
 }
@@ -1350,4 +1416,9 @@ SubversionControl *SubversionPlugin::subVersionControl() const
     return static_cast<SubversionControl *>(versionControl());
 }
 
-Q_EXPORT_PLUGIN(SubversionPlugin)
+} // Internal
+} // Subversion
+
+Q_EXPORT_PLUGIN(Subversion::Internal::SubversionPlugin)
+
+#include "subversionplugin.moc"
