@@ -6,26 +6,25 @@
 **
 ** Contact: Nokia Corporation (qt-info@nokia.com)
 **
-** No Commercial Usage
-**
-** This file contains pre-release code and may not be distributed.
-** You may use this file in accordance with the terms and conditions
-** contained in the Technology Preview License Agreement accompanying
-** this package.
 **
 ** GNU Lesser General Public License Usage
 **
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** This file may be used under the terms of the GNU Lesser General Public
+** License version 2.1 as published by the Free Software Foundation and
+** appearing in the file LICENSE.LGPL included in the packaging of this file.
+** Please review the following information to ensure the GNU Lesser General
+** Public License version 2.1 requirements will be met:
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Nokia gives you certain additional
-** rights.  These rights are described in the Nokia Qt LGPL Exception
+** rights. These rights are described in the Nokia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
+** Other Usage
+**
+** Alternatively, this file may be used in accordance with the terms and
+** conditions contained in a signed written agreement between you and Nokia.
+**
 ** If you have questions regarding the use of this file, please contact
 ** Nokia at qt-info@nokia.com.
 **
@@ -33,20 +32,22 @@
 
 #include "findinfiles.h"
 
+#include <coreplugin/editormanager/editormanager.h>
+
 #include <QtCore/QtDebug>
 #include <QtCore/QSettings>
 #include <QtCore/QDir>
 #include <QtCore/QDirIterator>
 #include <QtGui/QPushButton>
 #include <QtGui/QFileDialog>
+#include <QtGui/QHBoxLayout>
 #include <QtGui/QVBoxLayout>
 
 using namespace Find;
-using namespace TextEditor::Internal;
+using namespace TextEditor;
 
-FindInFiles::FindInFiles(SearchResultWindow *resultWindow)
-  : BaseFileFind(resultWindow),
-    m_configWidget(0),
+FindInFiles::FindInFiles()
+  : m_configWidget(0),
     m_directory(0)
 {
 }
@@ -67,10 +68,27 @@ void FindInFiles::findAll(const QString &txt, Find::FindFlags findFlags)
     BaseFileFind::findAll(txt, findFlags);
 }
 
-Utils::FileIterator *FindInFiles::files()
+Utils::FileIterator *FindInFiles::files() const
 {
-    return new Utils::SubDirFileIterator(QStringList() << m_directory->currentText(),
-                                         fileNameFilters());
+    return new Utils::SubDirFileIterator(QStringList() << QDir::fromNativeSeparators(m_directory->currentText()),
+                                         fileNameFilters(),
+                                         Core::EditorManager::instance()->defaultTextCodec());
+}
+
+QString FindInFiles::label() const
+{
+    const QStringList &nonEmptyComponents = QDir::cleanPath(
+                QFileInfo(QDir::fromNativeSeparators(m_directory->currentText())).absoluteFilePath())
+            .split(QLatin1Char('/'), QString::SkipEmptyParts);
+    return tr("Directory '%1':").arg(nonEmptyComponents.isEmpty() ? "/" : nonEmptyComponents.last());
+}
+
+QString FindInFiles::toolTip() const
+{
+    // %3 is filled by BaseFileFind::runNewSearch
+    return tr("Path: %1\nFilter: %2\n%3")
+            .arg(QDir::toNativeSeparators(QFileInfo(m_directory->currentText()).absoluteFilePath()))
+            .arg(fileNameFilters().join(QLatin1String(",")));
 }
 
 QWidget *FindInFiles::createConfigWidget()
@@ -81,7 +99,7 @@ QWidget *FindInFiles::createConfigWidget()
         gridLayout->setMargin(0);
         m_configWidget->setLayout(gridLayout);
 
-        QLabel *dirLabel = new QLabel(tr("&Directory:"));
+        QLabel *dirLabel = new QLabel(tr("Director&y:"));
         gridLayout->addWidget(dirLabel, 0, 0, Qt::AlignRight);
         m_directory = new QComboBox;
         m_directory->setEditable(true);
@@ -94,11 +112,11 @@ QWidget *FindInFiles::createConfigWidget()
         syncComboWithSettings(m_directory, m_directorySetting);
         dirLabel->setBuddy(m_directory);
         gridLayout->addWidget(m_directory, 0, 1);
-        QPushButton *browseButton = new QPushButton(tr("&Browse"));
+        QPushButton *browseButton = new QPushButton(tr("&Browse..."));
         gridLayout->addWidget(browseButton, 0, 2);
         connect(browseButton, SIGNAL(clicked()), this, SLOT(openFileBrowser()));
 
-        QLabel * const filePatternLabel = new QLabel(tr("File &pattern:"));
+        QLabel * const filePatternLabel = new QLabel(tr("Fi&le pattern:"));
         filePatternLabel->setMinimumWidth(80);
         filePatternLabel->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Preferred);
         filePatternLabel->setAlignment(Qt::AlignRight | Qt::AlignVCenter);
@@ -143,3 +161,9 @@ void FindInFiles::readSettings(QSettings *settings)
     settings->endGroup();
     syncComboWithSettings(m_directory, m_directorySetting);
 }
+
+void FindInFiles::setDirectory(const QString &directory)
+{
+    syncComboWithSettings(m_directory, directory);
+}
+
